#Comparison of FF and RF

#RF in R
rm(list=setdiff(ls(),"BIGSTART")) #REMOVEL ALL

#1) Import Data
data <-read.csv("./data_mining/cleaned_data/RRNews_episode_nocong2.csv")
data <-data[-1:-3,]
#2) Pre-process data
data <-data[5:nrow(data),]#remove the first 5 observations due to missing news information
cols<-c(data$sign_t.1,data$sign_t.2,data$sign_t.3,data$seconds_t.1,data$seconds_t,data$seconds_t.1,data$seconds_t.2,data$seconds_t.3)
cols<-as.factor(cols)
rownames(data)<-1:nrow(data)
data$news_t <- as.character(data$news_t)

#3) Import Data 2 (all news universe)
data2 <-read.csv("./data_mining/cleaned_data/allvdtv_final.csv")
data2 <-data2[-1:-3,]
data2 <-data2[5:nrow(data2),]#remove the first 5 observations due to missing news information
data2$news_t <- as.character(data2$news_t)

data$len<-nchar(data$news_t)
data2$len<-nchar(data2$news_t)


library(NLP)
library(tm)
#Estimate FRF (Requires randomForest, Wgcna and fuzzyforest)
#############RANDOM FOREST#################
library(randomForest)
######### FUZZY RANDOM FOREST #############
library(fuzzyforest)
library(foreach, doRNG)
library(WGCNA)
set.seed(7)


#####################################################################################################################
#ONE MONTH HORIZON
# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
tempdata<-data[1:(nrow(data)-1),c("news_t","episode_t.1.1","sign_t.1.1","len","year","month")] #one period ahead (the last obs I do not have info on t+1 conditional on X t to infer this relation so i removed it)
tempdata2<-data2[1:(nrow(data2)-1),c("news_t","len")]

# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
myCorpus1 <- Corpus(VectorSource(tempdata$news_t))
myCorpus2 <- Corpus(VectorSource(tempdata2$news_t))

# Bounds to build the numerator matrix (which is the leading)
mindf=ceiling(0.10*nrow(tempdata))
maxdf=ceiling(nrow(tempdata))
control=list(bounds=list(global=c(mindf,maxdf)))#,weighting=function(x) weightTfIdf(x, normalize = TRUE))


tdm1 <- TermDocumentMatrix(myCorpus1, control=control)
dictionary2<-as.character(tdm1$dimnames$Terms)
tdm2<-TermDocumentMatrix(myCorpus2,control=list(dictionary=dictionary2))
myCorpus1 <- NULL
myCorpus2<- NULL

# Compute relative frequency of terms of tdm1 wrt tdm2
m1<-as.matrix(tdm1,rownames.force=TRUE)
m2<-as.matrix(tdm2,rownames.force=TRUE)

m1b<-m1[order(rownames(m1)),]
rm(m1,tdm1,myCorpus1) 
m2b<-m2[order(rownames(m2)),]
rm(m2,tdm2,myCorpus2,dictionary2)

m3b<-round(m1b/m2b*100)
m3b[is.nan(m3b)] <- 0 

tdm<-as.TermDocumentMatrix(m3b, weighting = FALSE)
tdm<-removeSparseTerms(tdm, 0.5)
inspect(tdm)
rm(m2b,m1b,m3b,tempdata2)

wordMatrix <- as.data.frame( t(as.matrix(  tdm )))

################################################
# ALL DATA
y<-as.factor(tempdata$episode_t.1.1)
X<-wordMatrix

#1) Module learning using WGCNA (Default, you can customize your own)
#original configuration: minModuleSize<-30;drop_fraction <-.25 
net = blockwiseModules(X, power = 3, minModuleSize = 20,numericLabels = FALSE)
module_membership <- net$colors

#modules description
length(summary.factor(module_membership))
summary.factor(module_membership)
############################################################

mtry_factor <- 1/2; min_ntree <- 5000;  drop_fraction <- .01; ntree_factor <- 500
nodesize <- 1; final_ntree <- 10000; min_ntree2<-5000
mtry_factor2<- 1/2# I do this to control for the number of features from which to sample after screening
#before setting the param above better, hw, we could do better by increasing number of trees last step 
drop_fraction2 <- .01 #the smaller the more iterations it makes in the selection step
screen_params <- screen_control(drop_fraction = drop_fraction,
                                keep_fraction = .25, min_ntree = min_ntree,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor)
select_params <- select_control(drop_fraction = drop_fraction2,
                                number_selected = 50,
                                min_ntree = min_ntree2,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor) # number_selected = 500 if no df cuts of vocabulary is done


###############
#FUZZY FOREST: Prediction for P(EPISODE)
set.seed(7)
ff_fit <- ff(X, y, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata

rankings <- ff_fit$feature_list #selected features (generally biased)
final_rf <- ff_fit$final_rf #final rf fitted with selected features

Y_NEU<-rep(0,length(y))
Y_NEU[y==0]<-1

Y_POS<-rep(0,length(y))
Y_POS[y==1]<-1

y_real<-data.frame(Y_NEU,Y_POS)

#MORE MEASURES OF GOODNESS OF FIT
library(MLmetrics)
Accuracy(final_rf$predicted,y)
AUC(final_rf$votes, y)
GainAUC(final_rf$votes,y_real)
LiftAUC(final_rf$votes,y_real)
KS_Stat(final_rf$votes, y_real)
MultiLogLoss(final_rf$votes,y)
PRAUC(final_rf$votes,y_real)
Recall(y,final_rf$predicted)
ej<-final_rf$votes
mean(colMeans((y_real - ej)^2,na.rm=TRUE)) #mse
sqrt(sum(colMeans((y_real - ej)^2,na.rm=TRUE))) #rmse
sqrt(sum(colMeans((log(1 + y_real) - log(1 + ej))^2,na.rm = TRUE))) #RMSELE
ZeroOneLoss(final_rf$votes,y)
ZeroOneLoss(final_rf$votes,y_real)



###############
#RANDOM FOREST: Prediction for P(EPISODE)

#3) Estimate RF (Requires randomForest)
#############RANDOM FOREST#################
library(randomForest)
set.seed(7)
tuning<-tuneRF(X,y,ntreeTry = 10000,trace = TRUE, plot = TRUE, mtryStart = 3,nodesize=25)
set.seed(7)
fit <- randomForest(X,y,
                    importance=TRUE, 
                    ntree=10000,mtry=4,nodesize=15,replace=TRUE,strata=y,sampsize=c(452,20),oob.prox=TRUE,keep.forest = TRUE,proximity=TRUE)


###### GOODNESS OF FIT MEASURES
Y_NEU<-rep(0,length(y))
Y_NEU[y==0]<-1

Y_POS<-rep(0,length(y))
Y_POS[y==1]<-1

y_real<-data.frame(Y_NEU,Y_POS)

#MORE MEASURES OF GOODNESS OF FIT
library(MLmetrics)
Accuracy(final_rf$predicted,y)
AUC(final_rf$votes, y)
GainAUC(final_rf$votes,y_real)
LiftAUC(final_rf$votes,y_real)
KS_Stat(final_rf$votes, y_real)
MultiLogLoss(final_rf$votes,y)
PRAUC(final_rf$votes,y_real)
Recall(y,final_rf$predicted)
ej<-final_rf$votes
mean(colMeans((y_real - ej)^2,na.rm=TRUE)) #mse
sqrt(sum(colMeans((y_real - ej)^2,na.rm=TRUE))) #rmse
sqrt(sum(colMeans((log(1 + y_real) - log(1 + ej))^2,na.rm = TRUE))) #RMSELE
ZeroOneLoss(final_rf$votes,y)
ZeroOneLoss(final_rf$votes,y_real)



###################### OOB PLOTS

library(zoo)
library(ggplot2)
#FF
predoob<-final_rf$votes
predoob<-cbind(predoob,y)
#RF
predoobrf<-fit$votes
predoobrf<-cbind(predoobrf,y)

alloob <-data.frame(predoob,predoobrf,tempdata$year,tempdata$month)
colnames(alloob)<-c("ff0","ff1","yff","rf0","rf1","yrf","year","month")


alloob$timemonth<-alloob$year+(alloob$month-1)/12
alloob$mtime<-as.Date(yearmon(alloob$timemonth))

episodes11<-alloob$mtime[alloob$yff==2]


q<- ggplot(data = alloob, mapping = aes(x = mtime)) 
q + geom_line(aes(y = ff1), colour = "black",show.legend = TRUE) + 
  geom_vline(xintercept = episodes11, colour = "red",show.legend = TRUE) +
  geom_line(aes(y = rf1), colour = "blue",show.legend = TRUE) + 
  scale_x_date(date_breaks = "5 years" , date_labels = "%b-%y")+
  ylab("OOB Probability") + xlab("Time") 

ggsave("analysis/media_anticipation/results/oob_comparison.png", plot = last_plot(),  width=21, height = 12, units = "cm") #dpi="print",
