%% IMPULSE RESPONSE ESTIMATES OF TAX ANTICIPATIONS ON ECONOMIC ACTIVITY
% This code estimates the effects of tax anticipations and tax changes on
% economic activity. Generates impulse responses and standard error bands
% using semiparametric bootstrap procedure. The baseline MFDF part of this code
% is based on a code for provided by Gabriel Perez-Quirs in a times series
% master course at CEMFI. 
% Date of creation: 21-11-2019
clear
clc

global tax n vfq captst pnk vector filter;          % We specify which variables are global
global ny;
global lu la;

rng(7) % seed

%params
mira=0;f7=0;hs=0;H=0;F10=0;G=0;

va=1;      % va is the vare of rnd number to fill unobseved data@
vfq=1;     % Normalization of beta1 @
% lag length of the monthly-annual series@ 
pphi=2;    % pphi is the max lag length 
            % note: p, q1, and q2 <=pphi @
nk=pphi+1;  % nk is the first observation for which the
            % likelihood will be evaluated @
vector=[(1/3);(2/3);1;(2/3);(1/3)];  


je=0;      % je=0 to skip hessian calculations @
pes=1;     % pes=0 to skip weights calculations @
ks=0;
fore=0;    % fore=1 to compute forecasts @
vector=[(1/3);(2/3);1;(2/3);(1/3)];
ktr=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Using Vanderbilt TV Data
indica=readmatrix('../../../data/Economic Activity/usdata.xlsx','Range','B261:F733');
tax=readmatrix('../taxshocks_mr_lags.xlsx','Range','B285:CO757'); 
% FF TAX ANTICIPATIONS 
rf=readmatrix('../FF_sign_relnewst2_finalR.csv');
rf = rf(2:end,2:end);
rf2=rf(:,6:end); %tax rise anticipation (1,3) + tax cut anticipation (1,3)
rf=rf2;
tax=[tax,rf]; %merge tax changes and random forest predictions by date
m=mean(tax);

n = size(indica,2);

% calculate the mean and std dev without counting the dummy values (99999)
gg=indica(indica(:,1)~=99999,1);
megdp=mean(gg);
stdgdp=std(gg);
min1=min(gg);
max1=max(gg);
gg=indica(indica(:,2)~=99999,2);
meind1=mean(gg);
stdind1=std(gg);
min2=min(gg);
max2=max(gg);
gg=indica(indica(:,3)~=99999,3);
meind2=mean(gg);
stdind2=std(gg);
min3=min(gg);
max3=max(gg);
gg=indica(indica(:,4)~=99999,4);
meind3=mean(gg);
stdind3=std(gg);
min4=min(gg);
max4=max(gg);
gg=indica(indica(:,5)~=99999,5);
meind4=mean(gg);
stdind4=std(gg);
min5=min(gg);
max5=max(gg);
A= [megdp,stdgdp,min1,max1;meind1,stdind1,min2,max2;meind2,stdind2,min3,max3;meind3,stdind3,min4,max4;meind4,stdind4,min5,max5];

filename = '../../../analysis/desc/tableA6.xlsx';
writematrix(A,filename)


indica=demeaned(indica);
indica2=relleno(indica,1); % Fill missing values with a white noise draw
tax=demeaned(tax);

y=indica2;
ny=size(y,2);
cy=size(y,2)-2;
capt=size(y,1);

pnk=18; % pnk is the dimension of the state space (B) @                             
captst=capt-pphi;            % captst is the effective sample size @
yv=y(nk:capt,:);
tax=tax(nk:capt,:);

index=(indica(nk:capt,:)~=99999);

%/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%                Initial parameters' values
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

% LAGS(INCLUDING CURRENT T) AND HORIZONS TO CONSIDER

lu=36+1;
la=36+1;
ja=18;
ia=3*1;

ts=size(tax,2);

startval=[
0.267 
 0.497 
 0.604 
 0.430 
 0.381 
 0.423 
 0.255 
 0.694 
-0.576 
 0.197 
 0.521 
-0.196 
-0.159 
-0.423 
-0.222 
-0.156 
-0.003 
 0.252 
 0.502 
 0.514 
 0.741 
 0.873 
 rand(ts,1)
 ];
% 
% load paramsmr
% startval=[x;rand(size(rf,2),1)];
nth=length(startval);  % nth is number of parameters to be estimated @

%/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%         I will need the following matrices
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
filter=zeros(captst,pnk);  % Filter inferences @

H=0;F10=0;G=0;                            % Setting initial values (OBJECTS OF INTEREST)

%% Maximizing the likelihood function
options = optimset('DISPLAY','iter','TolFun',1e-8, 'MaxFunEvals',100000,'MaxIter',1000);
[x,fil]=fminunc(@(th)ofn_taxshocks(th,index,yv), startval, options);
%% Compute main IR
    
    A=x(6:7); 
    tauu=x(end-ts+1:end-ts+lu);
    taua=x(end-ts+1+lu:end-ts+lu+la);
    tauant=x(end-ja-6+1:end-6);
    prise=x(end-6+1:end-3);
    pcut=x(end-3+1:end); 
    IR = impulseresponAR2(A,tauu,taua,tauant,prise,pcut);
    save("main_linearMR",'IR','x','filter','tax', 'n', 'vfq', 'captst' ,'pnk','vector','lu','la','ja','ia','ts','startval','index','prise','pcut')

%% Bootstrap SE - Part I: data simulations
clear
clc
global vfq vector tax n captst pnk
load 'main_linearMR' 
lu=36+1;
la=36+1;
ja=18;
ia=3*1;
rng(7)
ts=size(tax,2);

%1) Given x_hat: Build matrices 
[R_hat,Q_hat,H_hat,F_hat,G_hat]=matrices_taxshocks(x);       % Given th, we obtain matrices R, Q, H, F. Fixed across simulations (take it out of here)

%2) Draw S points for each error term
S=1000;
mu = zeros(pnk,1);
indicaS=zeros(captst, n,S);
for s=1:S
    %generate simulated data
    W_hatS = mvnrnd(mu,Q_hat,captst);
    Y_s=simu_data_easy(W_hatS,H_hat,G_hat,F_hat,tax,filter)';    
    %Adjust series again
    indicaS(:,:,s)=Y_s;
end
save("simudata_linearMR",'indicaS','tax','index','captst','n','vector','vfq','pnk','lu','la','ia','ja','startval','ts')

%% Bootstrap SE - Part II: estimations
clear
clc
parpool(4)

load 'simudata_linearMR'
rng(7)
S=500;
RR = zeros(S,lu);
MRant = zeros(S,la+ja);
Medrise = zeros(S,3);
Medcut = Medrise;
Prise=zeros(S,3);
Pcut=zeros(S,3);
Taxmrise_del=zeros(S,3+ja+la);
Taxmcut_del=zeros(S,3+ja+la);
Taxmrise = zeros(S,3+lu);
Taxmcut = zeros(S,3+lu);

%3) Use s to generate data Ys and estimate again model and save IRs and
%effects.
options = optimset('TolFun',1e-8, 'MaxFunEvals',100000,'MaxIter',1000);
Xgrande = zeros(S,1);
tic
parfor s=1:S
    s
    % set initial values
    filter=zeros(captst,pnk);  % Filter inferences @
    H=0;F10=0;G=0;
    % estimate for simulated sample s (revisar globals)
    indica_s=squeeze(indicaS(:,:,s));
    [x_s,ff_s]=fminunc(@(th)ofn_taxshocks_simu_trial(th,index,indica_s,vfq, vector, tax, n, captst,pnk), startval, options);
    % compute IR for simulation s
    A_s=x_s(6:7); %ar(2) coefficients of space equation.
    tauu_s=x_s(end-ts+1:end-ts+lu);
    taua_s=x_s(end-ts+1+lu:end-ts+lu+la);
    tauant_s=x_s(end-ja-6+1:end-6);
    prise_s=x_s(end-6+1:end-3);
    pcut_s=x_s(end-3+1:end); 
    IR_s = impulseresponAR2trial(A_s,tauu_s,taua_s,tauant_s,prise_s,pcut_s,lu,la);
    
    % store results of each simulation
    RR(s,:)=IR_s.RR;
    MRant(s,:)=IR_s.MRant;
    Medrise(s,:)=IR_s.Medrise;
    Medcut(s,:)=IR_s.Medcut;
    Prise(s,:)=prise_s';
    Pcut(s,:)=pcut_s';
    Taxmrise_del(s,:) = IR_s.taxmrise_del;
    Taxmcut_del(s,:) = IR_s.taxmcut_del;
    Taxmrise(s,:) = IR_s.taxmrise;
    Taxmcut(s,:) = IR_s.taxmcut;          
    % Do it for all s in S and generate CI
end
toc

save("simuS_linearMR",'RR','MRant','Medrise','Medcut','Prise','Pcut','Taxmrise_del','Taxmcut_del','Taxmrise','Taxmcut');

%% Figures 
clear
clc
load 'simuS_linearMR'
load('main_linearMR','IR','prise','pcut')
lu=36+1;
la=36+1;
ja=18;
ia=3*1;

% Figure 5: Media Anticipation IRs
SE.Medrise = sqrt(var(Medrise));
LOWR = (IR.Medrise-SE.Medrise');
UPR = (IR.Medrise+SE.Medrise');

SE.Medcut = sqrt(var(Medcut));
LOWC = (IR.Medcut-SE.Medcut');
UPC= (IR.Medcut+SE.Medcut');
timeline=[-3:1:-1];


figure
subplot(1,2,1)
plot(timeline,IR.Medrise/2,'r')
hold on
plot(timeline,LOWR/2,'r--')
hold on
plot(timeline,UPR/2,'r--')
hold on
plot(timeline, zeros(ia,1), 'k')
hold on
ylim([-0.22 0.15])
plot([-ia -1; -ia -1], [ylim; ylim]', 'k')
xlabel('Months') 
ylabel('Percent') 
grid off
% title( 'Tax Increases')
xticks([-3 -2 -1])
hold off

% figure
subplot(1,2,2)
plot(timeline,IR.Medcut/2,'b')
hold on
plot(timeline,LOWC/2,'b--')
hold on
plot(timeline,UPC/2,'b--')
hold on
plot(timeline, zeros(ia,1), 'k')
hold on
ylim([-0.22 0.15])
plot([-ia -1; -ia -1], [ylim; ylim]', 'k')
xlabel('Months ') 
ylabel('Percent') 
grid off
% title('Tax Cuts')
xticks([-3 -2 -1])
hold off
set(gcf, 'Units', 'centimeters', 'Position', [0, 0, 25, 12])
saveas(gcf,'../../../analysis/tax_effects/results/fig5.png')

% Figure 6: Tax cuts impulse responses
% On-time
SE.Taxmcut = sqrt(var(Taxmcut));
LOWC = IR.taxmcut-SE.Taxmcut';
UPC= IR.taxmcut+SE.Taxmcut';
timeline=[-ia:1:lu-1];

figure
plot(timeline,IR.taxmcut,'b')
hold on
plot(timeline,LOWC,'b--')
hold on
plot(timeline,UPC,'b--')
hold on
plot(timeline, zeros(ia+lu,1), 'k--')
hold on
ylim([-1.5 2.5])
plot([0; 0], ylim','k')
hold off
%text(-3, 1.5, {'Pre-Approval'})
xlabel('Months') 
ylabel('Percent') 
grid off
axis tight
axis on
%title( 'Tax Multiplier for On-time Tax Changes')
set(gcf, 'Units', 'centimeters', 'Position', [0, 0, 18, 12])
saveas(gcf,'../../../analysis/tax_effects/results/fig6a.png')

% Delayed
SE.Taxmcut = sqrt(var(Taxmcut_del));
LOWC = IR.taxmcut_del-SE.Taxmcut';
UPC= IR.taxmcut_del+SE.Taxmcut';
timeline=[-ia-ja:1:la-1];

figure
plot(timeline,IR.taxmcut_del,'b')
hold on
plot(timeline,LOWC,'b--')
hold on
plot(timeline,UPC,'b--')
hold on
plot(timeline, zeros(ia+ja+la,1), 'k--')
hold on
ylim([-1.5 2.5])
plot([-18 0; -18 0], [ylim; ylim]', 'k')
hold off
%text([-18 0], 1.5*[1 1], {'Approval', 'Implementation'})
xlabel('Months') 
ylabel('Percent') 
grid off
axis tight
axis on
%title( 'Tax Multiplier for Delayed Tax Changes')
set(gcf, 'Units', 'centimeters', 'Position', [0, 0, 18, 12])
saveas(gcf,'../../../analysis/tax_effects/results/fig6b.png')


