%% IMPULSE RESPONSE ESTIMATES OF TAX ANTICIPATIONS ON ECONOMIC ACTIVITY
% This code estimates the effects of tax anticipations and tax changes on
% economic activity. It produces the robustness analysis which leaves one-month
% window for anticipations. Generates impulse responses and standard error bands
% using semiparametric bootstrap procedure. The baseline MFDF part of this code
% is based on a code for provided by Gabriel Perez-Quirs in a times series
% master course at CEMFI. 
% date: 21-11-2019
clear
clc

global tax n vfq captst pnk vector filter;          % We specify which variables are global
global ny;
global lu la;

rng(7)

%params
mira=0;f7=0;hs=0;H=0;F10=0;G=0;

va=1;      % va is the vare of rnd number to fill unobseved data@
vfq=1;     % Normalization of beta1 @
% lag length of the monthly-annual series@ 
pphi=2;    % pphi is the max lag length 
            % note: p, q1, and q2 <=pphi @
nk=pphi+1;  % nk is the first observation for which the
            % likelihood will be evaluated @
vector=[(1/3);(2/3);1;(2/3);(1/3)];  


je=0;      % je=0 to skip hessian calculations @
pes=1;     % pes=0 to skip weights calculations @
ks=0;
fore=0;    % fore=1 to compute forecasts @
vector=[(1/3);(2/3);1;(2/3);(1/3)];
ktr=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Using Vanderbilt TV Data
indica=readmatrix('../../../data/Economic Activity/usdata.xlsx','Range','B261:F733');
tax=readmatrix('../taxshocks_mr_lags.xlsx','Range','B285:CO757'); 
% SIGN-ANTICIPATED TAX EPISODES (remove 1mahead anticipations)
rf=readmatrix('../FF_sign_relnewst2_finalR.csv');
rf=rf(2:end,2:end);
rf2=rf(:,[7 8 10 11]); %tax rise anticipation (2,3) + tax cut anticipation (2,3)
rf=rf2;
tax=[tax,rf]; %merge tax changes and random forest predictions by date
m=mean(tax);

n = size(indica,2);

% calculate the mean and std dev without counting the dummy values (99999)
gg=indica(indica(:,1)~=99999,1);
megdp=mean(gg);
stdgdp=std(gg);
min1=min(gg);
max1=max(gg);
gg=indica(indica(:,2)~=99999,2);
meind1=mean(gg);
stdind1=std(gg);
min2=min(gg);
max2=max(gg);
gg=indica(indica(:,3)~=99999,3);
meind2=mean(gg);
stdind2=std(gg);
min3=min(gg);
max3=max(gg);
gg=indica(indica(:,4)~=99999,4);
meind3=mean(gg);
stdind3=std(gg);
min4=min(gg);
max4=max(gg);
gg=indica(indica(:,5)~=99999,5);
meind4=mean(gg);
stdind4=std(gg);
min5=min(gg);
max5=max(gg);
[megdp,stdgdp,min1,max1;meind1,stdind1,min2,max2;meind2,stdind2,min3,max3;meind3,stdind3,min4,max4;meind4,stdind4,min5,max5]


indica=demeaned(indica);
indica2=relleno(indica,1); % Fill missing values with a white noise draw
tax=demeaned(tax);

y=indica2;
ny=size(y,2);
cy=size(y,2)-2;
capt=size(y,1);

pnk=18; % pnk is the dimension of the state space (B) @                             
captst=capt-pphi;            % captst is the effective sample size @
yv=y(nk:capt,:);
tax=tax(nk:capt,:);

index=(indica(nk:capt,:)~=99999);

%/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%                Initial parameters' values
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

% LAGS(INCLUDING CURRENT T) AND HORIZONS TO CONSIDER

lu=36+1;
la=36+1;
ja=18;
ia=2*1; %1m window from 3 to 2

ts=size(tax,2);

startval=[
0.267 
 0.497 
 0.604 
 0.430 
 0.381 
 0.423 
 0.255 
 0.694 
-0.576 
 0.197 
 0.521 
-0.196 
-0.159 
-0.423 
-0.222 
-0.156 
-0.003 
 0.252 
 0.502 
 0.514 
 0.741 
 0.873 
 rand(ts,1)
 ];

nth=length(startval);  % nth is number of parameters to be estimated @

%/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%         I will need the following matrices
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
filter=zeros(captst,pnk);  % Filter inferences @

H=0;F10=0;G=0;                            % Setting initial values (OBJECTS OF INTEREST)

%% Maximizing the likelihood function
options = optimset('DISPLAY','iter','TolFun',1e-8, 'MaxFunEvals',100000,'MaxIter',1000);
[x,fil]=fminunc(@(th)ofn_taxshocks(th,index,yv), startval, options);
%% Compute main IR
    
    A=x(6:7); 
    tauu=x(end-ts+1:end-ts+lu);
    taua=x(end-ts+1+lu:end-ts+lu+la);
    tauant=x(end-ja-ia*2+1:end-ia*2);
    prise=x(end-ia*2+1:end-ia);
    pcut=x(end-ia+1:end); 
    IR = impulseresponAR2(A,tauu,taua,tauant,prise,pcut);
    save("1mwindow_linearMR",'IR','x','filter','tax', 'n', 'vfq', 'captst' ,'pnk','vector','lu','la','ja','ia','ts','startval','index','prise','pcut')

%% Bootstrap SE - Part I: data simulations
clear
clc
global vfq vector tax n captst pnk
load '1mwindow_linearMR' 
lu=36+1;
la=36+1;
ja=18;
ia=2*1;
rng(7)
ts=size(tax,2);

%1) Given x_hat: Build matrices 
[R_hat,Q_hat,H_hat,F_hat,G_hat]=matrices_taxshocks(x);       % Given th, we obtain matrices R, Q, H, F. Fixed across simulations (take it out of here)

%2) Draw S points for each error term
S=1000;
mu = zeros(pnk,1);
indicaS=zeros(captst, n,S);
for s=1:S
    %generate simulated data
    W_hatS = mvnrnd(mu,Q_hat,captst);
    Y_s=simu_data_easy(W_hatS,H_hat,G_hat,F_hat,tax,filter)';    
    %Adjust series again
    indicaS(:,:,s)=Y_s;
end
save("simudata_1mwlinearMR",'indicaS','tax','index','captst','n','vector','vfq','pnk','lu','la','ia','ja','startval','ts')

%% Bootstrap SE - Part II: estimations
clear
clc


load 'simudata_1mwlinearMR'
rng(7)
S=500;
RR = zeros(S,lu);
MRant = zeros(S,la+ja);
Medrise = zeros(S,ia);
Medcut = Medrise;
Prise=zeros(S,ia);
Pcut=zeros(S,ia);
Taxmrise_del=zeros(S,ia+ja+la);
Taxmcut_del=zeros(S,ia+ja+la);
Taxmrise = zeros(S,ia+lu);
Taxmcut = zeros(S,ia+lu);

%3) Use s to generate data Ys and estimate again model and save IRs and
%effects.
options = optimset('TolFun',1e-8, 'MaxFunEvals',100000,'MaxIter',1000);
Xgrande = zeros(S,1);
tic
parfor s=1:S
    s
    % set initial values
    filter=zeros(captst,pnk);  % Filter inferences @
    H=0;F10=0;G=0;
    % estimate for simulated sample s (revisar globals)
    indica_s=squeeze(indicaS(:,:,s));
    [x_s,ff_s]=fminunc(@(th)ofn_taxshocks_simu_trial(th,index,indica_s,vfq, vector, tax, n, captst,pnk), startval, options);
    % compute IR for simulation s
    A_s=x_s(6:7); %ar(2) coefficients of space equation.
    tauu_s=x_s(end-ts+1:end-ts+lu);
    taua_s=x_s(end-ts+1+lu:end-ts+lu+la);
    tauant_s=x_s(end-ja-ia*2+1:end-ia*2);
    prise_s=x_s(end-ia*2+1:end-ia);
    pcut_s=x_s(end-ia+1:end); 
    IR_s = impulseresponAR2trial(A_s,tauu_s,taua_s,tauant_s,prise_s,pcut_s,lu,la);
    
    % store results of each simulation
    RR(s,:)=IR_s.RR;
    MRant(s,:)=IR_s.MRant;
    Medrise(s,:)=IR_s.Medrise;
    Medcut(s,:)=IR_s.Medcut;
    Prise(s,:)=prise_s';
    Pcut(s,:)=pcut_s';
    Taxmrise_del(s,:) = IR_s.taxmrise_del;
    Taxmcut_del(s,:) = IR_s.taxmcut_del;
    Taxmrise(s,:) = IR_s.taxmrise;
    Taxmcut(s,:) = IR_s.taxmcut;          
    % Do it for all s in S and generate CI. (mean should converge too)
end
toc

save("simuS_1mwlinearMR",'RR','MRant','Medrise','Medcut','Prise','Pcut','Taxmrise_del','Taxmcut_del','Taxmrise','Taxmcut');

%% Figure A8
clear
clc
load 'simuS_1mwlinearMR'
load('1mwindow_linearMR','IR','prise','pcut')
lu=36+1;
la=36+1;
ja=18;
ia=3*1;

IRR = IR.taxmrise;
IRR = [IRR(1:2,:);IRR(2,:)];
SETaxmrise = sqrt(var(Taxmrise));
SETaxmrise = [SETaxmrise(:,1:2),SETaxmrise(:,2)];
LOWR = IRR-SETaxmrise';
UPR = IRR+SETaxmrise';

IRC = IR.taxmcut;
IRC = [IRC(1:2,:);IRC(2,:)];
SETaxmcut = sqrt(var(Taxmcut));
SETaxmcut = [SETaxmcut(:,1:2),SETaxmcut(:,2)];
LOWC = IRC-SETaxmcut';
UPC= IRC+SETaxmcut';
timeline=[-3:1:-1];

figure
subplot(1,2,1)
plot(timeline,IRR,'r')
hold on
plot(timeline,LOWR,'r--')
hold on
plot(timeline,UPR,'r--')
hold on
plot(timeline, zeros(ia,1), 'k')
hold on
ylim([-0.18 0.12])
plot([-ia -1; -ia -1], [ylim; ylim]', 'k')
xlabel('Months') 
ylabel('Percent') 
grid off
xticks([-3 -2 -1])
hold off

% figure
subplot(1,2,2)
plot(timeline,IRC,'b')
hold on
plot(timeline,LOWC,'b--')
hold on
plot(timeline,UPC,'b--')
hold on
plot(timeline, zeros(ia,1), 'k')
hold on
ylim([-0.18 0.12])
plot([-ia -1; -ia -1], [ylim; ylim]', 'k')
xlabel('Months ') 
ylabel('Percent') 
grid off
xticks([-3 -2 -1])
hold off
set(gcf, 'Units', 'centimeters', 'Position', [0, 0, 25, 12])
saveas(gcf,'../../../analysis/tax_effects/results/figA8.png')
