%% IMPULSE RESPONSE ESTIMATES OF TAX ANTICIPATIONS ON ECONOMIC ACTIVITY
% This code estimates the effects of tax anticipations and tax changes on
% economic activity. It controls for potentially different delay effects of 
% tax increases and tax cuts. Generates impulse responses and standard 
% error bands using semiparametric bootstrap procedure. 
% The baseline MFDF part of this code is based on a code for provided by Gabriel 
% Perez-Quirs in a times series master course at CEMFI. 
% date: 28-12-2019
clear
clc

global tax n vfq captst pnk vector filter;          % We specify which variables are global
global ny;
global lu la ja;

rng(7)

%params
mira=0;f7=0;hs=0;H=0;F10=0;G=0;

va=1;      % va is the vare of rnd number to fill unobseved data@
vfq=1;     % Normalization of beta1 @
% lag length of the monthly-annual series@ 
pphi=2;    % pphi is the max lag length 
            % note: p, q1, and q2 <=pphi @
nk=pphi+1;  % nk is the first observation for which the
            % likelihood will be evaluated @
vector=[(1/3);(2/3);1;(2/3);(1/3)];  


je=0;      % je=0 to skip hessian calculations @
pes=1;     % pes=0 to skip weights calculations @
ks=0;
fore=0;    % fore=1 to compute forecasts @
vector=[(1/3);(2/3);1;(2/3);(1/3)];
ktr=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Using Vanderbilt TV Data
indica=readmatrix('../../../data/Economic Activity/usdata.xlsx','Range','B261:F733');
tax=readmatrix('../taxshocks_mrsign_30lags12hor.xlsx','Range','B285:CI757'); 
% SIGN-ANTICIPATED TAX EPISODES
rf=readmatrix('../FF_sign_relnewst2_finalR.csv');
rf = rf(2:end,2:end);
rf2=rf(:,6:end); %tax rise anticipation (1,3) + tax cut anticipation (1,3)
rf=rf2;
tax=[tax,rf]; %merge tax changes and random forest predictions by date
m=mean(tax);
s=std(tax);

n = size(indica,2);

% calculate the mean and std dev without counting the dummy values (99999)
gg=indica(indica(:,1)~=99999,1);
megdp=mean(gg);
stdgdp=std(gg);
min1=min(gg);
max1=max(gg);
gg=indica(indica(:,2)~=99999,2);
meind1=mean(gg);
stdind1=std(gg);
min2=min(gg);
max2=max(gg);
gg=indica(indica(:,3)~=99999,3);
meind2=mean(gg);
stdind2=std(gg);
min3=min(gg);
max3=max(gg);
gg=indica(indica(:,4)~=99999,4);
meind3=mean(gg);
stdind3=std(gg);
min4=min(gg);
max4=max(gg);
gg=indica(indica(:,5)~=99999,5);
meind4=mean(gg);
stdind4=std(gg);
min5=min(gg);
max5=max(gg);
[megdp,stdgdp,min1,max1;meind1,stdind1,min2,max2;meind2,stdind2,min3,max3;meind3,stdind3,min4,max4;meind4,stdind4,min5,max5]

indica=demeaned(indica);
indica2=relleno(indica,1); % Fill missing values with a white noise draw
tax=demeaned(tax);

y=indica2;
ny=size(y,2);
cy=size(y,2)-2;
capt=size(y,1);

pnk=18; % pnk is the dimension of the state space (B) @                             
captst=capt-pphi;            % captst is the effective sample size @
yv=y(nk:capt,:);
tax=tax(nk:capt,:);

index=(indica(nk:capt,:)~=99999);

%/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%                Initial parameters' values
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

% LAGS(INCLUDING CURRENT T) AND HORIZONS TO CONSIDER

lu=30+1;
la=30+1;
ja=12;
ia=3*1;

ts=size(tax,2);

startval=[
0.267 
 0.497 
 0.604 
 0.430 
 0.381 
 0.423 
 0.255 
 0.694 
-0.576 
 0.197 
 0.521 
-0.196 
-0.159 
-0.423 
-0.222 
-0.156 
-0.003 
 0.252 
 0.502 
 0.514 
 0.741 
 0.873 
 rand(ts,1)
 ];

nth=length(startval);  % nth is number of parameters to be estimated @

%/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%         I will need the following matrices
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
filter=zeros(captst,pnk);  % Filter inferences @

H=0;F10=0;G=0;                            % Setting initial values (OBJECTS OF INTEREST)

%% Maximizing the likelihood function
options = optimset('DISPLAY','iter','TolFun',1e-8, 'MaxFunEvals',100000,'MaxIter',1000);
[x,fil]=fminunc(@(th)ofn_taxshocks(th,index,yv), startval, options);
%% Compute main IR
    
    A=x(6:7); 
    tauu=x(end-ts+1:end-ts+lu);
    taua=x(end-ts+1+lu:end-ts+lu+la);
    tauantrise=x(end-ts+lu+la+1:end-ts+lu+la+ja);
    tauantcut=x(end-ts+lu+la+ja+1:end-ts+lu+la+2*ja);
    prise=x(end-6+1:end-3);
    pcut=x(end-3+1:end); 
    IR = impulseresponAR2_nolin(A,tauu,taua,tauantrise,tauantcut,prise,pcut);
    
    save("main_nolinearMR12h",'IR','x','filter','tax', 'n', 'vfq', 'captst' ,'pnk','vector','lu','la','ja','ia','ts','startval','index','prise','pcut')

%% Bootstrap SE - Part I: data simulations
clear
clc
tic
global vfq vector tax n captst pnk
global lu la 
load 'main_nolinearMR12h' 
lu=30+1;
la=30+1;
ja=12;
ia=3*1;
rng(7)
ts=size(tax,2);
%1) Given x_hat: Build matrices 
[R_hat,Q_hat,H_hat,F_hat,G_hat]=matrices_taxshocks(x);       % Given th, we obtain matrices R, Q, H, F. Fixed across simulations (take it out of here)

%2) Draw S points for each error term
S=1000;
mu = zeros(pnk,1);
indicaS=zeros(captst, n,S);
for s=1:S
    %generate simulated data
    W_hatS = mvnrnd(mu,Q_hat,captst);
    Y_s=simu_data_easy(W_hatS,H_hat,G_hat,F_hat,tax,filter)';    
    %Adjust series again
    indicaS(:,:,s)=Y_s;
end
save("simudata_nolin12h",'indicaS','tax','index','captst','n','vector','vfq','pnk','lu','la','ia','ja','startval','ts')
%% Bootstrap SE - Part II: estimations
clear
clc

load 'simudata_nolin12h'
rng(7)
S=500;
RR = zeros(S,lu);
MRa = zeros(S,la);
MRrise = zeros(S,ja);
MRcut = MRrise;
Medrise = zeros(S,3);
Medcut = Medrise;
Prise=zeros(S,3);
Pcut=zeros(S,3);
Taxmrise_del=zeros(S,3+ja+la);
Taxmcut_del=zeros(S,3+ja+la);
Taxmrise = zeros(S,3+lu);
Taxmcut = zeros(S,3+lu);
%3) Use s to generate data Ys and estimate again model and save IRs and
%effects.
options = optimset('TolFun',1e-8, 'MaxFunEvals',100000,'MaxIter',1000);

tic
parfor s=1:S
    s
    % set initial values
    filter=zeros(captst,pnk);  % Filter inferences @
    H=0;F10=0;G=0;
    % estimate for simulated sample s (revisar globals)
    indica_s=squeeze(indicaS(:,:,s));
    [x_s,ff_s]=fminunc(@(th)ofn_taxshocks_simu_trial(th,index,indica_s,vfq, vector, tax, n, captst,pnk), startval, options);
    % compute IR for simulation s
    A_s=x_s(6:7); %ar(2) coefficients of space equation.
    tauu_s=x_s(end-ts+1:end-ts+lu);
    taua_s=x_s(end-ts+1+lu:end-ts+lu+la);
    tauantrise_s=x_s(end-ts+lu+la+1:end-ts+lu+la+ja);
    tauantcut_s=x_s(end-ts+lu+la+ja+1:end-ts+lu+la+2*ja);
    prise_s=x_s(end-6+1:end-3);
    pcut_s=x_s(end-3+1:end); 
    IR_s = impulseresponAR2_nolin_trial(A_s,tauu_s,taua_s,tauantrise_s,tauantcut_s,prise_s,pcut_s,lu,la);
    % store results of each simulation
    RR(s,:)=IR_s.RR;
    MRa(s,:)=IR_s.MRa;
    MRrise(s,:)=IR_s.MRrise;
    MRcut(s,:)=IR_s.MRcut;
    Medrise(s,:)=IR_s.Medrise;
    Medcut(s,:)=IR_s.Medcut;
    Prise(s,:)=prise_s';
    Pcut(s,:)=pcut_s';
    Taxmrise_del(s,:) = IR_s.taxmrise_del;
    Taxmcut_del(s,:) = IR_s.taxmcut_del;
    Taxmrise(s,:) = IR_s.taxmrise;
    Taxmcut(s,:) = IR_s.taxmcut;    
    % Do it for all s in S and generate CI. 
end
save("simuS_nolinearMR12h",'RR','MRa','MRrise','MRcut','Medrise','Medcut','Prise','Pcut','Taxmrise_del','Taxmcut_del','Taxmrise','Taxmcut');
toc

%% Figure 8: Anticipations + Non-linear Delays 
clear
clc
load 'simuS_nolinearMR12h'
load('main_nolinearMR12h','IR','prise','pcut')
lu=30+1;
la=30+1;
ja=12;
ia=3*1;

SE.Taxmrise = sqrt(var(Taxmrise_del));
LOWR = IR.taxmrise_del-SE.Taxmrise';
UPR = IR.taxmrise_del+SE.Taxmrise';

SE.Taxmcut = sqrt(var(Taxmcut_del));
LOWC = IR.taxmcut_del-SE.Taxmcut';
UPC= IR.taxmcut_del+SE.Taxmcut';
timeline=[-ia-ja:1:-1];

figure
plot(timeline,IR.taxmrise_del(1:ia+ja),'b','LineWidth',1)
hold on
plot(timeline,LOWR(1:ia+ja),'b--','LineWidth',1)
hold on
plot(timeline,UPR(1:ia+ja),'b--','LineWidth',1)
hold on
plot(timeline,IR.taxmcut_del(1:ia+ja),'b','LineWidth',2.5)
hold on
plot(timeline,LOWC(1:ia+ja),'b--','LineWidth',2.5)
hold on
plot(timeline,UPC(1:ia+ja),'b--','LineWidth',2.5)
hold on
plot(timeline, zeros(ia+ja,1), 'k')
hold on
ylim([-2 1.5])
plot([-ja ; -ja], [ylim; ylim]', 'k')
xlabel('Months') 
ylabel('Percent') 
grid off
axis tight
set(gcf, 'Units', 'centimeters', 'Position', [0, 0, 18, 12])
saveas(gcf,'../../../analysis/tax_effects/results/fig8.png')

