% VAR model manual
% Date: 2019-09-26
% Author: Garcia-Uribe
% Paper: The Effects of Tax Changes on Economic Activity: A Narrative
% Approach to Frequent Anticipations.

%% Import Data ( 4 variables ) 
clear
clc
global lu la ja ia p n

% Response data WINDOWS
tic
raw = readmatrix('../../../data/Economic Activity/usdata.xlsx','Range','A261:Q733');
num = raw(:,2:end);

% Main - Figure 9
series = [num(:,2:6)];

lu=36+1;
la=36+1;
ja=18;
ia=3*1;
p=1;

% Import exogenous variables 
RRtax=readmatrix('../taxshocks_mr_lags.xlsx','Range','B285:CO757'); %tax data from 1/8/1968
% SIGN-ANTICIPATED TAX EPISODES
FFant=readmatrix('../FF_sign_relnewst2_finalR.csv');
FFant = FFant(2:end,2:end);
FFant=FFant(:,end-6+1:end);

X = [RRtax,FFant];
Ylag = lagmatrix(series,p);

DX = [ones(size(Ylag,1),1),Ylag,X];
DY = series;

DX = DX(p+1:end,:)';
DY = DY(p+1:end,:)';

[n,T] = size(DY);

% Obtain reduced-form errors
beta = DY*((DX*DX')'\DX)';   %VECM coefficients
res = (DY-beta*DX)';
Sigma  = res'*res/(T-p*n-1);
B = chol(Sigma,'lower');

A0= reshape(beta(:,1),n,1); % constant
A=beta(:,2:n+1); %AR(1) coefficients
tauu=beta(:,1+n+1:1+n+lu);
taua=beta(:,1+n+lu+1:1+n+lu+la);
tauant=beta(:,1+n+lu+la+1:1+n+lu+la+ja);
prise=beta(:,1+n+lu+la+ja+1:1+n+lu+la+ja+3);
pcut=beta(:,1+n+lu+la+ja+3+1:end);
exog=[tauu,taua,tauant,prise,pcut];

IR=impulserespon(A,tauu,taua,tauant,prise,pcut);

%% Bootstrap CI
S=10000;
irRRS=zeros(S,lu,n);
irMRantS=zeros(S,la+ja,n);
irMedriseS=zeros(S,ia,n);
irMedcutS=zeros(S,ia,n);
irMEDS=zeros(S,6,n);
for s=1:S
% Sample with replacement from residuals
eb   =  res(randsample(length(res),length(res),true),:);
resb =  eb';
% Compute Y, given X, eb and "true" coefficients, recursively.
yb  = zeros(n,T+1);
yb(:,1:p)=series(1:p,:)';
for t=1+p:T 
     yb(:,t) = A0+resb(:,t-p); %add the simulated residual
     yb(:,t) = yb(:,t)+exog*X(t,:)'; % X es de la misma dimension que ser yb.
     yb(:,t) = yb(:,t)+A*yb(:,t-p); %Add the impact of each of the lagged coeff     
end
% Restimate the model with new yb and X.

Ylag = lagmatrix(yb',p);

DX = [ones(size(Ylag,1),1),Ylag,X];
DY = yb';

DX = DX(p+1:end,:)';
DY = DY(p+1:end,:)';

[n,T] = size(DY);

% Obtain reduced-form errors
beta_s = DY*((DX*DX')'\DX)';   %VECM coefficients
res_s = (DY-beta_s*DX)';
Sigma_s  = res_s'*res_s/(T-p*n-1);
Bs = chol(Sigma_s,'lower');

A0_s= reshape(beta_s(:,1),n,1); % constant
A_s=beta_s(:,2:n+1); %AR(1) coefficients
tauu_s=beta_s(:,1+n+1:1+n+lu);
taua_s=beta_s(:,1+n+lu+1:1+n+lu+la);
tauant_s=beta_s(:,1+n+lu+la+1:1+n+lu+la+ja);
prise_s=beta_s(:,1+n+lu+la+ja+1:1+n+lu+la+ja+3);
pcut_s=beta_s(:,1+n+lu+la+ja+3+1:end);

% Simulate impulse responses
MEDs=[prise_s,pcut_s];
IRs=impulserespon(A_s,tauu_s,taua_s,tauant_s,prise_s,pcut_s);
irRRS(s,:,:)=IRs.irRR';
irMRantS(s,:,:)=IRs.irMRant';
irMedriseS(s,:,:)=IRs.irMedrise';
irMedcutS(s,:,:)=IRs.irMedcut';
irMEDS(s,:,:)=MEDs';
end
toc
%% Confidence interval for each series (noncentered?)
clevel=68;

for nn=1:n
    IRCIL.irRR(nn,:)=quantile(irRRS(:,:,nn),(1-clevel/100)/2);
    IRCIU.irRR(nn,:)=quantile(irRRS(:,:,nn),1-(1-clevel/100)/2);
    IRCIL.irMRant(nn,:)=quantile(irMRantS(:,:,nn),(1-clevel/100)/2);
    IRCIU.irMRant(nn,:)=quantile(irMRantS(:,:,nn),1-(1-clevel/100)/2);
    IRCIL.irMedrise(nn,:)=quantile(irMedriseS(:,:,nn),(1-clevel/100)/2);
    IRCIU.irMedrise(nn,:)=quantile(irMedriseS(:,:,nn),1-(1-clevel/100)/2);
    IRCIL.irMedcut(nn,:)=quantile(irMedcutS(:,:,nn),(1-clevel/100)/2);
    IRCIU.irMedcut(nn,:)=quantile(irMedcutS(:,:,nn),1-(1-clevel/100)/2);
    irMEDSCIU(nn,:)=quantile(irMEDS(:,:,nn),1-(1-clevel/100)/2);
    irMEDSCIL(nn,:)=quantile(irMEDS(:,:,nn),(1-clevel/100)/2);
    IRMED(nn,:)=mean(irMEDS(:,:,nn));
    IRRR(nn,:) = IR.irRR(nn,:);
    IRMRant(nn,:) = IR.irMRant(nn,:);
    IRMEDRISE(nn,:)=IR.irMedrise(nn,:);
    IRMEDCUT(nn,:)=IR.irMedcut(nn,:);

end

%% PLOTS
% Main Figure 9
figure
subplot(5,2,1)
plot([-3:1:-1],IR.irMedrise(1,1:3)/2,'r')
hold on
plot([-3:1:-1],IRCIL.irMedrise(1,:)/2,'r--')
hold on
plot([-3:1:-1],IRCIU.irMedrise(1,:)/2,'r--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.5 0.4])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('IPI')
xlabel('Months') 
ylabel('Percent') 
xticks([-3 -2 -1])
grid on
subplot(5,2,2)
plot([-3:1:-1],IR.irMedcut(1,:)/2,'b')
hold on
plot([-3:1:-1],IRCIL.irMedcut(1,:)/2,'b--')
hold on
plot([-3:1:-1],IRCIU.irMedcut(1,:)/2,'b--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.5 0.4])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('IPI')
xlabel('Months') 
ylabel('Percent') 
xticks([-3 -2 -1])
grid on
subplot(5,2,3)
plot([-3:1:-1],IR.irMedrise(2,:)/2,'r')
hold on
plot([-3:1:-1],IRCIL.irMedrise(2,:)/2,'r--')
hold on
plot([-3:1:-1],IRCIU.irMedrise(2,:)/2,'r--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.2 0.2])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('Employment')
xlabel('Months') 
ylabel('Percent') 
xticks([-3 -2 -1])
grid on
subplot(5,2,4)
plot([-3:1:-1],IR.irMedcut(2,:)/2,'b')
hold on
plot([-3:1:-1],IRCIL.irMedcut(2,:)/2,'b--')
hold on
plot([-3:1:-1],IRCIU.irMedcut(2,:)/2,'b--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.2 0.2])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('Employment')
xlabel('Months') 
ylabel('Percent')
xticks([-3 -2 -1])
grid on
subplot(5,2,5)
plot([-3:1:-1],IR.irMedrise(3,:)/2,'r')
hold on
plot([-3:1:-1],IRCIL.irMedrise(3,:)/2,'r--')
hold on
plot([-3:1:-1],IRCIU.irMedrise(3,:)/2,'r--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.8 0.8])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('Manufactures')
xlabel('Months') 
ylabel('Percent') 
xticks([-3 -2 -1])
grid on
subplot(5,2,6)
plot([-3:1:-1],IR.irMedcut(3,:)/2,'b')
hold on
plot([-3:1:-1],IRCIL.irMedcut(3,:)/2,'b--')
hold on
plot([-3:1:-1],IRCIU.irMedcut(3,:)/2,'b--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.8 0.8])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('Manufactures')
xlabel('Months') 
ylabel('Percent')
xticks([-3 -2 -1])
grid on
subplot(5,2,7)
plot([-3:1:-1],IR.irMedrise(4,:)/2,'r')
hold on
plot([-3:1:-1],IRCIL.irMedrise(4,:)/2,'r--')
hold on
plot([-3:1:-1],IRCIU.irMedrise(4,:)/2,'r--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.35 0.35])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('Income')
xlabel('Months') 
ylabel('Percent') 
xticks([-3 -2 -1])
grid on
subplot(5,2,8)
plot([-3:1:-1],IR.irMedcut(4,:)/2,'b')
hold on
plot([-3:1:-1],IRCIL.irMedcut(4,:)/2,'b--')
hold on
plot([-3:1:-1],IRCIU.irMedcut(4,:)/2,'b--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.35 0.35])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('Income')
xlabel('Months') 
ylabel('Percent')
xticks([-3 -2 -1])
grid on
subplot(5,2,9)
plot([-3:1:-1],IR.irMedrise(5,:)/2,'r')
hold on
plot([-3:1:-1],IRCIL.irMedrise(5,:)/2,'r--')
hold on
plot([-3:1:-1],IRCIU.irMedrise(5,:)/2,'r--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.4 0.4])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('PCE')
xlabel('Months') 
ylabel('Percent') 
xticks([-3 -2 -1])
grid on
subplot(5,2,10)
plot([-3:1:-1],IR.irMedcut(5,:)/2,'b')
hold on
plot([-3:1:-1],IRCIL.irMedcut(5,:)/2,'b--')
hold on
plot([-3:1:-1],IRCIU.irMedcut(5,:)/2,'b--')
hold on
plot([-3 -1], [0 0], 'k')
hold on
ylim([-0.4 0.4])
plot([-3 -1; -3 -1],[ylim; ylim]','k')
hold off
title('PCE')
xlabel('Months') 
ylabel('Percent')
xticks([-3 -2 -1])
grid on
set(gcf, 'Units', 'centimeters', 'Position', [0, 0, 14, 27])
saveas(gcf,'../../../analysis/tax_effects/results/fig9.png')
