# -*- coding: utf-8 -*-
"""
Created on Tue Feb 21 12:39:01 2017

@author: Sandra García-Uribe

TEXT DATA PREPROCESSING TO USE IN RANDOM FOREST

"""

import pandas as pd
import os, unicodedata
import csv,sys
csv.field_size_limit(12000000) #sys.maxsize within brackets  
import numpy as np

def remove_nnp(text):
    cleaned=[]
    for sentence in parse(text,tokenize=True,tags=True, chunks=True).split():
        for part in sentence:
            if (part[1].startswith('NNP') or part[1].startswith('NNPS')) and (part[0] in dictionary):
              cleaned.append(part[0])      
            if not (part[1].startswith('NNP') or part[1].startswith('NNPS')):
              cleaned.append(part[0])
    fin=' '.join(cleaned)
    return fin   

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False
        
def lemmatizer(cleaned):
    #It parses the tokens, retrieves tags, assigns a reduction to each tag (VERB,NOUN,ADJ)
    lemmatized = []
    for w in cleaned:
        tag = parse(w,tagset='parole').split()[0][0][1]
        fl = w
        #noun --> Singularize
        if tag.startswith('N')==True: fl = singularize(w)
        #verb --> Lemmatize
        if tag.startswith('V')==True: fl = lemma(w)
        #adj --> Singularize
        if tag.startswith('A')==True: fl = singularize(w)
        #Otherwise, adverb, pronoun: Keep the original.
        lemmatized.append(fl)
    # Remove stopwords after lemmatization since some appear again
    lemmasc=[stemmer.stem(lem) for lem in lemmatized  if ((lem not in stopwords) and (is_number(lem)==False)  and (lem!='') and (len(lem)>2))]
    lemmatized = ' '.join(lemmasc)    
    return lemmatized

from nltk.stem.porter import *
stemmer = PorterStemmer()  

from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer, TfidfTransformer 
def tokenize(text):
    tokens = nltk.word_tokenize(text)
    output = [token for token in tokens]
    return output
def preprocess(corpus):    
    vectorizer= CountVectorizer(tokenizer=tokenize, encoding='utf-8', stop_words=None, lowercase=True, preprocessor=None, strip_accents='unicode')# ,min_df=6,max_df=400
    counts = vectorizer.fit_transform(corpus)
    vocab = vectorizer.get_feature_names()
    return vocab,counts  


os.chdir(ORPATH+EXPATH)

# Aggregate data to month level
news=pd.read_csv('tvnews_nocongressmen.csv',header=0,encoding='utf-8-sig')
news=news.rename(columns = {'0':'year'})
news=news.rename(columns = {'1':'month'})
news=news.rename(columns = {'2':'day'})

news['date']=pd.to_datetime(news.year*10000+news.month*100+news.day,format='%Y%m%d')
news['date']=news['date'].astype('datetime64[ns]')
news['9']=news['9'].replace(np.nan,"", regex=True)
news['seconds']=news['14'].replace(np.nan,0,regex=True)

#Clean text data to proceed to vectorization afterwards ( remove stopwords, punctuation, headings)
from nltk.corpus import stopwords
import numpy as np
stopwords=stopwords.words('english')
##1st lowercase
news['9']=news['9'].replace(np.nan,' ', regex=True)
news[10]=news['9'].apply(lambda x: str(unicodedata.normalize('NFKD', unicode(x)).encode('ASCII', 'ignore')))
##############################################
from pattern.en import parse,singularize,lemma
from collections import Counter
import numpy as np
##############################################
news[11]=news[10].apply(lambda x: x.lower())
#generate the dummy for "tax increase" as opposed to "tax cut"
import re
regex=re.compile('increase|increases|\srise|\srises')
news['taxrise']=news[11].apply(lambda x: (re.search(regex,x)!=None))
news['taxrise']=news['taxrise'].apply(lambda x: int(x == True))
##2nd remove all is in parenthesis 
import re
news[12]=news[11].apply(lambda x: re.sub('\(.*?\)','', x))
##3rd Remove stopwords, numbers, punctuation
import nltk, string
news[12]=news[12].apply(lambda x: str(unicodedata.normalize('NFKD', unicode(x)).encode('ASCII', 'ignore')))
news[13]=news[12].apply(lambda x: x.translate(None,string.punctuation))
#substitute tokens that contain digits and delete tokens with lesss than 3 characters.        
news[13]=news[13].apply(lambda x: [re.sub('[0-9]','',el) for el in nltk.word_tokenize(x)])
news[13]=news[13].apply(lambda x: [el for el in x if el!=''])      
#Part of Speech-Processing (includes removal of stopwords)  and Stemming
news[14]=news[13].apply(lambda x: lemmatizer(x))

#agg functions
def jointext(texto):
    return ','.join(texto)
def sumatodo(elementos):
    return sum(elementos)
def maxvalue(elementos):
    return max(elementos)
news1=news.set_index('date')
newsm=news1.groupby(['month','year'],as_index=False)   
newsm=newsm.agg({14: jointext, 'seconds': sumatodo, 'taxrise': maxvalue})


#Gnerate balanced monthly news
import itertools
years=range(1968,2008)
months=range(1,13)
rectangle = list(itertools.product(years, months))
aux=pd.DataFrame(rectangle)
aux.columns = ['year','month']
balanced=aux.merge(newsm,how='left')

#Merge to episodes data
episodes=pd.read_excel('episodedata_signsize.xls',header=0)
merged = balanced.merge(episodes,how='left')
merged.taxrise=merged.taxrise.replace(np.nan,0,regex=True)

#Create lagged data with pd.concat(s, s.shift()],axis=1)
merged[15]=merged[14].shift()
merged[16]=merged[15].shift()
merged[17]=merged[16].shift()
merged[18]=merged[17].shift()
merged[19]=merged[18].shift()
merged[20]=merged[19].shift()

merged=merged.rename(columns = {14:'news_t'})
merged=merged.rename(columns = {15:'news_t-1'})
merged=merged.rename(columns = {16:'news_t-2'})
merged=merged.rename(columns = {17:'news_t-3'})
merged=merged.rename(columns = {18:'news_t-4'})
merged=merged.rename(columns = {19:'news_t-5'})
merged=merged.rename(columns = {20:'news_t-6'})

merged[21]=merged['sign'].shift(-1)
merged[22]=merged['sign'].shift(-2)
merged[23]=merged['sign'].shift(-3)
merged[24]=merged['sign'].shift(-4)
merged[25]=merged['sign'].shift(-5)
merged[26]=merged['sign'].shift(-6)


merged=merged.rename(columns = {21:'sign_t+1'})
merged=merged.rename(columns = {22:'sign_t+2'})
merged=merged.rename(columns = {23:'sign_t+3'})
merged=merged.rename(columns = {24:'sign_t+4'})
merged=merged.rename(columns = {25:'sign_t+5'})
merged=merged.rename(columns = {26:'sign_t+6'})



merged['seconds']=merged['seconds'].replace(np.nan,0,regex=True)

merged[27]=merged['seconds'].shift(1)
merged[28]=merged['seconds'].shift(2)
merged[29]=merged['seconds'].shift(3)
merged[30]=merged['seconds'].shift(4)
merged[31]=merged['seconds'].shift(5)
merged[32]=merged['seconds'].shift(6)

merged[33]=merged['episode'].shift(1)
merged[34]=merged['episode'].shift(2)
merged[35]=merged['episode'].shift(3)
merged[36]=merged['episode'].shift(4)
merged[37]=merged['episode'].shift(5)
merged[38]=merged['episode'].shift(6)

merged[39]=merged['sign'].shift(1)
merged[40]=merged['sign'].shift(2)
merged[41]=merged['sign'].shift(3)

merged[42]=merged['episode'].shift(-1)
merged[43]=merged['episode'].shift(-2)
merged[44]=merged['episode'].shift(-3)


merged[45]=merged['sign'].shift(-1)
merged[46]=merged['sign'].shift(-2)
merged[47]=merged['sign'].shift(-3)

merged=merged.rename(columns = {27:'seconds_t-1'})
merged=merged.rename(columns = {28:'seconds_t-2'})
merged=merged.rename(columns = {29:'seconds_t-3'})
merged=merged.rename(columns = {30:'seconds_t-4'})
merged=merged.rename(columns = {31:'seconds_t-5'})
merged=merged.rename(columns = {32:'seconds_t-6'})
merged=merged.rename(columns = {'seconds':'seconds_t'})

merged=merged.rename(columns = {33:'episode_t-1'})
merged=merged.rename(columns = {34:'episode_t-2'})
merged=merged.rename(columns = {35:'episode_t-3'})
merged=merged.rename(columns = {36:'episode_t-4'})
merged=merged.rename(columns = {37:'episode_t-5'})
merged=merged.rename(columns = {38:'episode_t-6'})


merged=merged.rename(columns = {39:'sign_t-1'})
merged=merged.rename(columns = {40:'sign_t-2'})
merged=merged.rename(columns = {41:'sign_t-3'})

merged=merged.rename(columns = {42:'episode_t+1'})
merged=merged.rename(columns = {43:'episode_t+2'})
merged=merged.rename(columns = {44:'episode_t+3'})

merged[45]=merged['idsize'].shift(-1)
merged[46]=merged['idsize'].shift(-2)
merged[47]=merged['idsize'].shift(-3)

merged=merged.rename(columns = {45:'ss_t+1'})
merged=merged.rename(columns = {46:'ss_t+2'})
merged=merged.rename(columns = {47:'ss_t+3'})

#Sign is known three periods ahead
merged['sign_t+1']=merged['sign_t+1'].replace(np.nan,0,regex=True)
merged['sign_t+2']=merged['sign_t+2'].replace(np.nan,0,regex=True)
merged['sign_t+3']=merged['sign_t+3'].replace(np.nan,0,regex=True)

#Size-sign split
merged['ss_t+1']=merged['ss_t+1'].replace(np.nan,0,regex=True)
merged['ss_t+2']=merged['ss_t+2'].replace(np.nan,0,regex=True)
merged['ss_t+3']=merged['ss_t+3'].replace(np.nan,0,regex=True)

final=merged[['year','month','episode','episode_t-1','episode_t-2','episode_t-3','episode_t+1','episode_t+2','episode_t+3','sign_t-1','sign_t-2','sign_t-3','sign','sign_t+1','sign_t+2','sign_t+3','Episode_name','id','news_t','news_t-1','news_t-2','news_t-3','seconds_t','seconds_t-1','seconds_t-2','seconds_t-3','taxrise','ss_t+1','ss_t+2','ss_t+3']]
final['agnews']=final['news_t']+''+final['news_t-1']+''+final['news_t-2']+''+final['news_t-3']

##############################################################################
# Save Data
final.to_csv("RRNews_episode_nocong2.csv",sep=',',quoting=csv.QUOTE_MINIMAL)
