# -*- coding: utf-8 -*-
"""
Created on 24-7-2019
@author: Sandra Garcia-Uribe
PREPROCESSING ALL NEWS DATA TO COMPUTE RELATIVE FREQUENCY OF TERMS FOR RANDOM FORESTS

"""
import pandas as pd
import os, unicodedata
import csv,sys
csv.field_size_limit(12000000) #sys.maxsize within brackets  
import numpy as np

def remove_nnp(text):
    cleaned=[]
    for sentence in parse(text,tokenize=True,tags=True, chunks=True).split():
        for part in sentence:
            if (part[1].startswith('NNP') or part[1].startswith('NNPS')) and (part[0] in dictionary):
              cleaned.append(part[0])      
            if not (part[1].startswith('NNP') or part[1].startswith('NNPS')):
              cleaned.append(part[0])
    fin=' '.join(cleaned)
    return fin   

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False
        
def lemmatizer(cleaned):
    #It parses the tokens, retrieves tags, assigns a reduction to each tag (VERB,NOUN,ADJ)
    lemmatized = []
    for w in cleaned:
        tag = parse(w,tagset='parole').split()[0][0][1]
        fl = w
        #noun --> Singularize
        if tag.startswith('N')==True: fl = singularize(w)
        #verb --> Lemmatize
        if tag.startswith('V')==True: fl = lemma(w)
        #adj --> Singularize
        if tag.startswith('A')==True: fl = singularize(w)
        #Otherwise, adverb, pronoun: Keep the original.
        lemmatized.append(fl)
    # Remove stopwords after lemmatization since some appear again
    lemmasc=[stemmer.stem(lem) for lem in lemmatized  if ((lem not in stopwords) and (is_number(lem)==False)  and (lem!='') and (len(lem)>2))]
    lemmatized = ' '.join(lemmasc)    
    return lemmatized

from nltk.stem.porter import *
stemmer = PorterStemmer()  

from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer, TfidfTransformer 
def tokenize(text):
    tokens = nltk.word_tokenize(text)
    output = [token for token in tokens]
    return output
def preprocess(corpus):    
    vectorizer= CountVectorizer(tokenizer=tokenize, encoding='utf-8', stop_words=None, lowercase=True, preprocessor=None, strip_accents='unicode')# ,min_df=6,max_df=400
    counts = vectorizer.fit_transform(corpus)
    vocab = vectorizer.get_feature_names()
    return vocab,counts  

os.chdir(ORPATH+EXPATH)

# Aggregate data to month level
news=pd.read_csv('allvdtv_0.csv',header=0,encoding='utf-8-sig')
news=news.rename(columns = {'0':'year'})
news=news.rename(columns = {'1':'month'})
news=news.rename(columns = {'2':'day'})

news['date']=pd.to_datetime(news.year*10000+news.month*100+news.day,format='%Y%m%d')
news['date']=news['date'].astype('datetime64[ns]')
news['9']=news['9'].replace(np.nan,"", regex=True)
news['seconds']=news['14'].replace(np.nan,0,regex=True)

#Clean text data to proceed to vectorization afterwards ( remove stopwords, punctuation, headings)
from nltk.corpus import stopwords
import numpy as np
stopwords=stopwords.words('english')
##1st lowercase
news['9']=news['9'].replace(np.nan,' ', regex=True)
news[10]=news['9'].apply(lambda x: str(unicodedata.normalize('NFKD', unicode(x)).encode('ASCII', 'ignore')))
##############################################
from pattern.en import parse,singularize,lemma
from collections import Counter
import numpy as np
##############################################
news[11]=news[10].apply(lambda x: x.lower())
##2nd remove all is in parenthesis 
import re
news[12]=news[11].apply(lambda x: re.sub('\(.*?\)','', x))
##3rd Remove stopwords, numbers, punctuation
import nltk, string
news[12]=news[12].apply(lambda x: str(unicodedata.normalize('NFKD', unicode(x)).encode('ASCII', 'ignore')))
news[13]=news[12].apply(lambda x: x.translate(None,string.punctuation))
#substitute tokens that contain digits and delete tokens with lesss than 3 characters.        
news[13]=news[13].apply(lambda x: [re.sub('[0-9]','',el) for el in nltk.word_tokenize(x)])
news[13]=news[13].apply(lambda x: [el for el in x if el!=''])      
#Part of Speech-Processing (includes removal of stopwords)  and Stemming
news[14]=news[13].apply(lambda x: lemmatizer(x))


#agg functions
def jointext(texto):
    return ','.join(texto)
def sumatodo(elementos):
    return sum(elementos)
news1=news.set_index('date')
newsm=news1.groupby(['month','year'],as_index=False)   
newsm=newsm.agg({14: jointext, 'seconds': sumatodo})


#Gnerate balanced monthly news
import itertools
years=range(1968,2008)
months=range(1,13)
rectangle = list(itertools.product(years, months))
aux=pd.DataFrame(rectangle)
aux.columns = ['year','month']
balanced=aux.merge(newsm,how='left')

merged = balanced

merged[15]=merged[14].shift()
merged[16]=merged[15].shift()
merged[17]=merged[16].shift()

merged=merged.rename(columns = {14:'news_t'})
merged=merged.rename(columns = {15:'news_t-1'})
merged=merged.rename(columns = {16:'news_t-2'})

merged['seconds']=merged['seconds'].replace(np.nan,0,regex=True)

#Export
final=merged[['year','month','news_t']]
##############################################################################
# Save Data
final.to_csv("allvdtv_final.csv",sep=',',quoting=csv.QUOTE_MINIMAL)
