/*Monthly Exogenous Tax Shocks*/
clear

/*Auxiliary Monthly Database 1*/
clear
set obs 64
gen obs=_n
gen year=1944+obs
foreach num of numlist 1/12{
gen yearf`num'=.
}
reshape long yearf, i(year) j(month)
drop yearf
rename year ys
rename month ms
sort ys ms

save "temp/aux_dates1.dta",replace

/*Auxiliary Monthly Database 2*/
clear
set obs 64
gen obs=_n
gen year=1944+obs
foreach num of numlist 1/12{
gen yearf`num'=.
}
reshape long yearf, i(year) j(month)
drop yearf
gen auxdate=ym(year,month)
format auxdate %tm
sort auxdate

save "temp/aux_dates2.dta",replace

/*Quarterly GDP to normalize tax changes*/
cd ..
import excel "data/Economic Activity/usqgdp.xlsx", firstrow clear
gen im=mofd(observation_date)
format im %tm
rename GDP QGDP

keep im QGDP
sort im

save "data_mining/temp/qgdp.dta", replace


********************************************************************
/*TAX SHOCKS*/
import excel "data/Tax changes/romer_tax_data.xlsx", sheet("Exog_all") firstrow clear

/*id for law*/
encode Episode_name, generate(id)

/*Keep Retroactive Values When Applicable*/
bysort id Episode_retro: gen rid=_n==1
bysort id: egen sr=sum(rid)
drop if sr==2 & Episode_retro=="no"
*replace Episode_signed=Episode_implemented if Episode_retro=="yes"

/*id for shock*/
sort Episode_signed
gen idchange=_n


/*month dates*/
gen ys=floor(Episode_signed/10000)
gen ms=floor(Episode_signed/100)
replace ms=ms-ys*100
gen ds=Episode_signed-(ys*10000+ms*100)

gen yi=floor(Episode_implemented/10000)
gen mi=floor(Episode_implemented/100)
replace mi=mi-yi*100
gen di=Episode_implemented-(yi*10000+mi*100)

gen yss=string(ys)
gen mss=string(ms,"%02.0f")
gen sdate=yss+mss

gen dates=mdy(ms,ds,ys)
gen sm = mofd(dates)
format sm %tm
format dates %td

gen yis=string(yi)
gen mis=string(mi,"%02.0f")
gen idate=yis+mis

gen datei=mdy(mi,di,yi)
gen im = mofd(datei)
format im %tm
format datei %td


gen anticipated=0
replace anticipated=1 if datei-dates>10
count if anticipated==0
br id idchange sm im if anticipated==0

gen horizon=0
replace horizon=im-sm if anticipated==1
*hist horizon, bin(81)

*NO WINDOW AT ALL
/*gen latepassage=0
replace latepassage=1 if ds>=25 & dates!=.

gen mso=ms
replace ys=ys+1 if latepassage==1 & mso==12
replace ms=1 if latepassage==1 & mso==12
replace ms=ms+1 if latepassage==1 & mso!=12
replace ds=1 if latepassage==1*/

replace dates=mdy(ms,ds,ys)
replace sm = mofd(dates)
format sm %tm
format dates %td

save "data_mining/temp/episodedata.dta", replace

/*Standardize by current quarter GDP*/
sort im
merge im using "data_mining/temp/qgdp.dta"
drop _merge
gen Episode_change_gdp=Episode_change/QGDP*100*3

bys sm: egen cumchange=sum(Episode_change_gdp) /*cummulative of changes by signature date because of your predictive approach is by anticipation of signature*/
gen sign=sign(cumchange) 


bys sm: egen aux=min(idchange)
drop if aux!=idchange | aux==. /*end with an signaturedate database (there may be, rarely, two episodes signed the same date but we will aggregate them*/
gen episode=1

sort ys ms
merge ys ms using "data_mining/temp/aux_dates1.dta"
rename _merge _merge1
br ys ms id idchange yi mi horizon	
sort ys ms

keep ys ms sign episode Episode_name id
replace episode=0 if episode==.
replace sign=0 if sign==.

rename ys year
rename ms month

order year month episode sign Episode_name
export excel using "data_mining/cleaned_data/episodedata_RR2robustness.xls", firstrow(variables) replace 

