/*THIS CODE PRODUCES VARIOUS DATASETS WITH MONTHLY TIME SERIES OF EXOGENOUS TAX 
CHANGES AND RELATED MEASURES OF TAX CHANGES THAT WILL BE USED AT THE CREATION OF 
ANTICIPATION MEASURE AND THE EFFECTS OT TAX CHANGES ON ECONOMIC ACTIVITY*/

/*By changing lu la and hor maximum one can generate different tax datasets for estimation*/

/*In this version we distinguish between tax increases delays and tax cuts delays*/


/*Monthly Exogenous TAx Shocks*/
/*Auxiliary Monthly Database 1*/
clear
set obs 64
gen obs=_n
gen year=1944+obs
foreach num of numlist 1/12{
gen yearf`num'=.
}
reshape long yearf, i(year) j(month)
drop yearf
rename year ys
rename month ms
sort ys ms

save "temp/aux_dates1.dta",replace

/*Auxiliary Monthly Database 2*/
clear
set obs 64
gen obs=_n
gen year=1944+obs
foreach num of numlist 1/12{
gen yearf`num'=.
}
reshape long yearf, i(year) j(month)
drop yearf
gen auxdate=ym(year,month)
format auxdate %tm
sort auxdate

save "temp/aux_dates2.dta",replace

/*Quarterly GDP to normalize tax changes*/
cd ..
import excel "data/Economic Activity/usqgdp.xlsx", firstrow clear
gen im=mofd(observation_date)
format im %tm
rename GDP QGDP

keep im QGDP
sort im

save "data_mining/temp/qgdp.dta", replace

********************************************************************
/*TAX SHOCKS*/
import excel "data/Tax changes/romer_tax_data.xlsx", sheet("Exog_all") firstrow clear

/*id for law*/
encode Episode_name, generate(id)

/*Ignore retroactive components of bills*/
drop if Episode_retro=="yes" 
/*id for shock*/
sort Episode_signed
gen idchange=_n


/*month dates*/
gen ys=floor(Episode_signed/10000)
gen ms=floor(Episode_signed/100)
replace ms=ms-ys*100
gen ds=Episode_signed-(ys*10000+ms*100)

gen yi=floor(Episode_implemented/10000)
gen mi=floor(Episode_implemented/100)
replace mi=mi-yi*100
gen di=Episode_implemented-(yi*10000+mi*100)

gen yss=string(ys)
gen mss=string(ms,"%02.0f")
gen sdate=yss+mss

gen dates=mdy(ms,ds,ys)
gen sm = mofd(dates)
format sm %tm
format dates %td 

gen yis=string(yi)
gen mis=string(mi,"%02.0f")
gen idate=yis+mis

gen datei=mdy(mi,di,yi)
gen im = mofd(datei)
format im %tm
format datei %td

sort ys ms
gen latepassage=0
replace latepassage=1 if ds>=20 & dates!=.

gen mso=ms
replace ys=ys+1 if latepassage==1 & mso==12
replace ms=1 if latepassage==1 & mso==12
replace ms=ms+1 if latepassage==1 & mso!=12
replace ds=1 if latepassage==1

replace dates=mdy(ms,ds,ys)
replace sm = mofd(dates)
format sm %tm
format dates %td

sort yi mi
gen lateimplement=0
replace lateimplement=1 if di>=20 & datei!=.

gen mio=mi
replace yi=yi+1 if lateimplement==1 & mio==12
replace mi=1 if lateimplement==1 & mio==12
replace mi=mi+1 if lateimplement==1 & mio!=12
replace di=1 if lateimplement==1

replace datei=mdy(mi,di,yi)
replace im = mofd(datei)
format im %tm
format datei %td


gen anticipated=0
replace anticipated=1 if datei-dates>10
count if anticipated==0
br id idchange sm im if anticipated==0

gen horizon=0
replace horizon=im-sm if anticipated==1
hist horizon, bin(81)

save "data_mining/temp/episodedata_sign.dta", replace

/*Standardize by corresponding quarterly GDP  */ 
sort im
merge im using "data_mining/temp/qgdp.dta"
drop _merge
gen Episode_change_gdp=Episode_change/QGDP*100 /* Episode_change=Tax_change (misleading original wording)*/
bys id: egen cumchange=sum(Episode_change_gdp)
gen sign=sign(cumchange)

save "data_mining/temp/episodedata_balanced_forimp_sign.dta", replace

sort ys ms
merge ys ms using "data_mining/temp/aux_dates1.dta"
rename _merge _merge1
br ys ms id idchange yi mi horizon anticipated	sign
sort ys ms

save "data_mining/temp/episodedata_balanced_sign.dta", replace

*expand the dataset to the maximum anticipation horizon in the data
*after you will truncate this anticipation horizon

sum horizon
expand r(max)
sort ys ms idchange 

bys idchange: gen auxdate=sm+_n-1
*bys sm: gen auxdate=sm+_n-1
format auxdate %tm
br sm id idchange im auxdate horizon	
tab idchange

drop if auxdate>im
tab idchange
gen hort=im-auxdate

sort auxdate hort idchange
br auxdate hort id idchange sm im

drop if _merge==2 /*once you drop you get a horizonspan-monthdate dataset*/
drop _merge
count 
count if anticipated==0
br auxdate hort id idchange sm im if anticipated==1

bys auxdate hort: gen cums=_N
*so that you do not consider unanticipated shocks for the anticipated measures
gen sign_change=sign(Episode_change_gdp)
gen Episode_change_aux_pos=Episode_change_gdp
replace Episode_change_aux_pos=0 if anticipated==0 | sign_change==-1

gen Episode_change_aux_neg=Episode_change_gdp
replace Episode_change_aux_neg=0 if anticipated==0 | sign_change==1

br auxdate hort id idchange sm im cums Episode_change_gdp Episode_change_aux* anticipated sign_change

local M=12
foreach hor of numlist 0/`M'{
bysort auxdate hort: egen tax_a_pos_`hor'=sum(Episode_change_aux_pos) 
replace tax_a_pos_`hor'=0 if hort!=`hor' 
bysort auxdate: egen max_tax_a_pos_`hor'=max(tax_a_pos_`hor')
bysort auxdate: egen min_tax_a_pos_`hor'=min(tax_a_pos_`hor')
egen _tax_a_pos_`hor'=rowtotal(max_tax_a_pos_`hor' min_tax_a_pos_`hor') 
replace _tax_a_pos_`hor'=max_tax_a_pos_`hor' if min_tax_a_pos_`hor'==max_tax_a_pos_`hor'
}

local M=12
foreach hor of numlist 0/`M'{
bysort auxdate hort: egen tax_a_neg_`hor'=sum(Episode_change_aux_neg) 
replace tax_a_neg_`hor'=0 if hort!=`hor' 
bysort auxdate: egen max_tax_a_neg_`hor'=max(tax_a_neg_`hor')
bysort auxdate: egen min_tax_a_neg_`hor'=min(tax_a_neg_`hor')
egen _tax_a_neg_`hor'=rowtotal(max_tax_a_neg_`hor' min_tax_a_neg_`hor') 
replace _tax_a_neg_`hor'=max_tax_a_neg_`hor' if min_tax_a_neg_`hor'==max_tax_a_neg_`hor'
}
drop if hort==1 & anticipated==0
sort auxdate
merge auxdate using "data_mining/temp/aux_dates2.dta"

sort auxdate hort im idchange
br auxdate hort cums idchange sm im Episode_change_gdp tax*

br auxdate hort cums anticipated idchange sm im Episode_change_gdp Episode_change_aux* tax* if anticipated==0

gen Episode_change_un=0
replace Episode_change_un=Episode_change_gdp if anticipated==0 & im==auxdate

*drop tax_a*
*rename mtax_a* tax_a*

*agregate unanticipated tax changes in case there are multiple per period
bys auxdate anticipated: egen tax_u=sum(Episode_change_un)
bys auxdate: egen min_tax_u=min(tax_u)
bys auxdate: egen max_tax_u=max(tax_u)
egen _tax_u=rowtotal(max_tax_u min_tax_u) 
replace _tax_u=max_tax_u if min_tax_u==max_tax_u

gen _tax_a_0=_tax_a_pos_0+_tax_a_neg_0
drop _tax_a_pos_0 _tax_a_neg_0
replace _tax_u=0 if anticipated==1
replace _tax_a_0=0 if anticipated==0

bys auxdate: gen od=_n==1

drop if od==0

sort auxdate
keep auxdate _tax*

foreach var of varlist _tax*{
replace `var'=0 if `var'==.
}

gen _tax_rr=_tax_u+_tax_a_0

/*Export Monthly Tax Shocks Ready to Input to Factor Model*/
sort auxdate
local lu=30
foreach lag of numlist 1/`lu'{
gen _tax_u_`lag' = _tax_u[_n-`lag'] if auxdate==auxdate[_n-`lag']+`lag'
}
local la=30
foreach lag of numlist 1/`la'{
*gen _tax_a_0_`lag'=L(`lag')._tax_a_0
gen _tax_a_0_`lag'= _tax_a_0[_n-`lag'] if auxdate==auxdate[_n-`lag']+`lag'
}

order auxdate _tax_u _tax_u_* _tax_a_0 _tax_a_0_* _tax_a_*
export excel using "analysis/tax_effects/taxshocks_mrsign_30lags12hor.xlsx", firstrow(variables) replace

