#RF in R
rm(list=setdiff(ls(),"BIGSTART")) #REMOVEL ALL
#1) Import Data
data <-read.csv("./data_mining/cleaned_data/RRNews_episode_nocong2.csv")
data <-data[-1:-3,]
#2) Pre-process data
data <-data[5:nrow(data),]#remove the first 5 observations due to missing news information
cols<-c(data$sign_t.1,data$sign_t.2,data$sign_t.3,data$seconds_t.1,data$seconds_t,data$seconds_t.1,data$seconds_t.2,data$seconds_t.3)
cols<-as.factor(cols)
rownames(data)<-1:nrow(data)
data$news_t <- as.character(data$news_t)

#3) Import Data 2 (all news universe)
data2 <-read.csv("./data_mining/cleaned_data/allvdtv_final.csv")
data2 <-data2[-1:-3,]
data2 <-data2[5:nrow(data2),]#remove the first 5 observations due to missing news information
data2$news_t <- as.character(data2$news_t)

data$len<-nchar(data$news_t)
data2$len<-nchar(data2$news_t)


library(NLP)
library(tm)
#Estimate FRF (Requires randomForest, Wgcna and fuzzyforest)
#############RANDOM FOREST#################
library(randomForest)
######### FUZZY RANDOM FOREST #############
library(fuzzyforest)
library(foreach, doRNG)
library(WGCNA)
set.seed(7)


#####################################################################################################################
#ONE MONTH HORIZON
# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
tempdata<-data[1:(nrow(data)-1),c("news_t","episode_t.1.1","sign_t.1.1","len","year","month")] #one period ahead (the last obs I do not have info on t+1 conditional on X t to infer this relation so i removed it)
tempdata2<-data2[1:(nrow(data2)-1),c("news_t","len")]

# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
myCorpus1 <- Corpus(VectorSource(tempdata$news_t))
myCorpus2 <- Corpus(VectorSource(tempdata2$news_t))

# Bounds to build the numerator matrix (which is the leading)
mindf=ceiling(0.10*nrow(tempdata))
maxdf=ceiling(nrow(tempdata))
control=list(bounds=list(global=c(mindf,maxdf)))#,weighting=function(x) weightTfIdf(x, normalize = TRUE))


tdm1 <- TermDocumentMatrix(myCorpus1, control=control)
dictionary2<-as.character(tdm1$dimnames$Terms)
tdm2<-TermDocumentMatrix(myCorpus2,control=list(dictionary=dictionary2))
myCorpus1 <- NULL
myCorpus2<- NULL

# Compute relative frequency of terms of tdm1 wrt tdm2
m1<-as.matrix(tdm1,rownames.force=TRUE)
m2<-as.matrix(tdm2,rownames.force=TRUE)

m1b<-m1[order(rownames(m1)),]
rm(m1,tdm1,myCorpus1) 
m2b<-m2[order(rownames(m2)),]
rm(m2,tdm2,myCorpus2,dictionary2)

m3b<-round(m1b/m2b*100)
m3b[is.nan(m3b)] <- 0 

tdm<-as.TermDocumentMatrix(m3b, weighting = FALSE)
tdm<-removeSparseTerms(tdm, 0.5)
inspect(tdm)
rm(m2b,m1b,m3b,tempdata2)

wordMatrix <- as.data.frame( t(as.matrix(  tdm )))

################################################
# ALL DATA
y<-as.factor(tempdata$episode_t.1.1)
X<-wordMatrix
# SIGN+ subset 1
subset1<-which(tempdata$sign_t.1.1==1 | tempdata$sign_t.1.1 ==0)
# SIGN- subset 2
subset2<-which(tempdata$sign_t.1.1==-1 | tempdata$sign_t.1.1==0)


#1) Module learning using WGCNA (Default, you can customize your own)
#original configuration: minModuleSize<-30;drop_fraction <-.25 
net = blockwiseModules(X, power = 3, minModuleSize = 20,numericLabels = FALSE)
module_membership <- net$colors

#modules description
length(summary.factor(module_membership))
summary.factor(module_membership)
############################################################

mtry_factor <- 1/2; min_ntree <- 5000;  drop_fraction <- .01; ntree_factor <- 500
nodesize <- 1; final_ntree <- 10000; min_ntree2<-5000
mtry_factor2<- 1/2# I do this to control for the number of features from which to sample after screening
#before setting the param above better, hw, we could do better by increasing number of trees last step 
drop_fraction2 <- .01 #the smaller the more iterations it makes in the selection step
screen_params <- screen_control(drop_fraction = drop_fraction,
                                keep_fraction = .25, min_ntree = min_ntree,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor)
select_params <- select_control(drop_fraction = drop_fraction2,
                                number_selected = 50,
                                min_ntree = min_ntree2,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor) # number_selected = 500 if no df cuts of vocabulary is done

vocab<-tdm$dimnames$Terms

###############
#Prediction for P(EPISODE)
set.seed(7)
ff_fit <- ff(X, y, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata

rankings <- ff_fit$feature_list #selected features (generally biased)
final_rf <- ff_fit$final_rf #final rf fitted with selected features

varImpPlot(final_rf,main = "") #final rf variable importance

#variable importance plot
importance(final_rf)
png('analysis/media_anticipation/results/sign/ff_news.png',width = 8, height = 7, units = 'in', res = 800)
varImpPlot(final_rf,main=" ")
dev.off()

varimp<-final_rf$importance
varimp<-data.frame(varimp)
rownames(varimp)[order(-varimp$MeanDecreaseAccuracy)]
rownames(varimp)[order(-varimp$MeanDecreaseGini)]

#Train Predicted Probability
predprob<-predict(final_rf,X,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y)
#store results in the following variables
horizon1<-tempdata
horizon1$predprob0<-0 #for all
horizon1$predprob1<-0 #for subset1
horizon1$predprob2<-0 #for subset2

horizon1$predoob0<-0 #for all
horizon1$predoob1<-0 #for subset1
horizon1$predoob2<-0 #for subset2

horizon1$predprob0<-predprob[,2]
horizon1$predoob0<-predoob[,2]

###############
# Prediction of tax rise (sign +)
y1<-y[subset1]
X1<-X[subset1,]
set.seed(7)
ff_fit <- ff(X1, y1, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X1,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y1)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y1)
#store results
horizon1$predprob1[subset1]<-predprob[,2]
horizon1$predoob1[subset1]<-predoob[,2]

varImpPlot(final_rf,main = "") #final rf variable importance

#variable importance plot
importance(final_rf)
png('analysis/media_anticipation/results/sign/ff_news_inc.png',width = 8, height = 7, units = 'in', res = 800)
varImpPlot(final_rf,main=" ")
dev.off()

###############
# Prediction of tax cut (sign -)
y2<-y[subset2]
X2<-X[subset2,]
set.seed(7)
ff_fit <- ff(X2, y2, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X2,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y2)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y2)
#store results
horizon1$predprob2[subset2]<-predprob[,2]
horizon1$predoob2[subset2]<-predoob[,2]

varImpPlot(final_rf,main = "") #final rf variable importance

#variable importance plot
importance(final_rf)
png('analysis/media_anticipation/results/sign/ff_news_cut.png',width = 8, height = 7, units = 'in', res = 800)
varImpPlot(final_rf,main=" ")
dev.off()

lastline<-rep(0, ncol(horizon1))
horizon1<-rbind(horizon1,lastline)
horizon1$p1<-horizon1$predprob1/(1-horizon1$predprob1)*(1-horizon1$predprob0)
horizon1$p2<-horizon1$predprob2/(1-horizon1$predprob2)*(1-horizon1$predprob0)
horizon1$oob1<-horizon1$predoob1/(1-horizon1$predoob1)*(1-horizon1$predoob0)
horizon1$oob2<-horizon1$predoob2/(1-horizon1$predoob2)*(1-horizon1$predoob0)



library(zoo)
library(ggplot2)
subset11<-tempdata$sign_t.1.1
subset11[subset11==-1]<-0
subset22<-tempdata$sign_t.1.1
subset22[subset22==1]<-0
subset22[subset22==-1]<-1

pincoob<-horizon1$oob1[1:(nrow(horizon1)-1)]
pcutoob<-horizon1$oob2[1:(nrow(horizon1)-1)]
pincprob<-horizon1$predprob1[1:(nrow(horizon1)-1)]
pcutprob<-horizon1$predprob2[1:(nrow(horizon1)-1)]
final_data<- data.frame(pincoob,pcutoob,pincprob,pcutprob,subset11,subset22,tempdata$year,tempdata$month) 

final_data$timemonth<-final_data$tempdata.year+(final_data$tempdata.month-1)/12
final_data$mtime<-as.Date(yearmon(final_data$timemonth))
episodes11<-final_data$mtime[final_data$subset11==1]
episodes22<-final_data$mtime[final_data$subset22==1]

q<- ggplot(data = final_data, mapping = aes(x = mtime)) 
q + geom_line(aes(y = pincoob), colour = "black",show.legend = TRUE) + 
  geom_vline(xintercept = episodes11, colour = "red",show.legend = TRUE) +
  scale_x_date(date_breaks = "5 years" , date_labels = "%b-%y")+
  ylab("OOB Probability of a Tax Rise Bill at t+1") + xlab("Time") 

ggsave("analysis/media_anticipation/results/sign/oobinc.png", plot = last_plot(),  width=21, height = 12, units = "cm") #dpi="print",

q<- ggplot(data = final_data, mapping = aes(x = mtime)) 
q + geom_line(aes(y = pcutoob), colour = "black",show.legend = TRUE) + 
  geom_vline(xintercept = episodes22, colour = "red",show.legend = TRUE) +
  scale_x_date(date_breaks = "5 years" , date_labels = "%b-%y")+
  ylab("OOB Probability of a Tax Cut Approval at t+1") + xlab("Time")

ggsave("analysis/media_anticipation/results/sign/oobcut.png", plot = last_plot(),  width=21, height = 12, units = "cm") #dpi="print",

q<- ggplot(data = final_data, mapping = aes(x = mtime)) 
q + geom_line(aes(y = pincprob), colour = "black",show.legend = TRUE) + 
  geom_vline(xintercept = episodes11, colour = "red",show.legend = TRUE) +
  scale_x_date(date_breaks = "5 years" , date_labels = "%b-%y")+
  ylab("Fitted Probability of a Tax Rise Bill at t+1") + xlab("Time") 

ggsave("analysis/media_anticipation/results/sign/fitinc.png", plot = last_plot(),  width=21, height = 12, units = "cm") #dpi="print",

q<- ggplot(data = final_data, mapping = aes(x = mtime)) 
q + geom_line(aes(y = pcutprob), colour = "black",show.legend = TRUE) + 
  geom_vline(xintercept = episodes22, colour = "red",show.legend = TRUE) +
  scale_x_date(date_breaks = "5 years" , date_labels = "%b-%y")+
  ylab("Fitted Probability of a Tax Cut Approval at t+1") + xlab("Time")

ggsave("analysis/media_anticipation/results/sign/fitcut.png", plot = last_plot(),  width=21, height = 12, units = "cm") #dpi="print",



###############################################################
# TWO MONTH HORIZON
# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
tempdata<-data[1:(nrow(data)-2),c("news_t","episode_t.2.1","sign_t.2.1","len")] #one period ahead (the last obs I do not have info on t+1 conditional on X t to infer this relation so i removed it)
tempdata2<-data2[1:(nrow(data2)-2),c("news_t","len")]

# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
myCorpus1 <- Corpus(VectorSource(tempdata$news_t))
myCorpus2 <- Corpus(VectorSource(tempdata2$news_t))

# Bounds to build the numerator matrix (which is the leading)
mindf=ceiling(0.10*nrow(tempdata))
maxdf=ceiling(nrow(tempdata))
control=list(bounds=list(global=c(mindf,maxdf)))#,weighting=function(x) weightTfIdf(x, normalize = TRUE))


tdm1 <- TermDocumentMatrix(myCorpus1, control=control)
dictionary2<-as.character(tdm1$dimnames$Terms)
tdm2<-TermDocumentMatrix(myCorpus2,control=list(dictionary=dictionary2))
myCorpus1 <- NULL
myCorpus2<- NULL

# Compute relative frequency of terms of tdm1 wrt tdm2
m1<-as.matrix(tdm1,rownames.force=TRUE)
m2<-as.matrix(tdm2,rownames.force=TRUE)

m1b<-m1[order(rownames(m1)),]
rm(m1,tdm1,myCorpus1) 
m2b<-m2[order(rownames(m2)),]
rm(m2,tdm2,myCorpus2,dictionary2)

m3b<-round(m1b/m2b*100)
m3b[is.nan(m3b)] <- 0 

tdm<-as.TermDocumentMatrix(m3b, weighting = FALSE)
tdm<-removeSparseTerms(tdm, 0.5)
inspect(tdm)
rm(m2b,m1b,m3b,tempdata2)

wordMatrix <- as.data.frame( t(as.matrix(  tdm )))

################################################
# ALL DATA
y<-as.factor(tempdata$episode_t.2.1)
X<-wordMatrix
# SIGN+ subset 1
subset1<-which(tempdata$sign_t.2.1==1 | tempdata$sign_t.2.1 ==0)
# SIGN- subset 2
subset2<-which(tempdata$sign_t.2.1==-1 | tempdata$sign_t.2.1==0)

#1) Module learning using WGCNA (Default, you can customize your own)
#original configuration: minModuleSize<-30;drop_fraction <-.25 
net = blockwiseModules(X, power = 3, minModuleSize = 20,numericLabels = FALSE)
module_membership <- net$colors

#modules description
length(summary.factor(module_membership))
summary.factor(module_membership)
############################################################

mtry_factor <- 1/2; min_ntree <- 5000;  drop_fraction <- .01; ntree_factor <- 500
nodesize <- 1; final_ntree <- 10000; min_ntree2<-5000
mtry_factor2<- 1/2# I do this to control for the number of features from which to sample after screening
#before setting the param above better, hw, we could do better by increasing number of trees last step 
drop_fraction2 <- .01 #the smaller the more iterations it makes in the selection step
screen_params <- screen_control(drop_fraction = drop_fraction,
                                keep_fraction = .25, min_ntree = min_ntree,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor)
select_params <- select_control(drop_fraction = drop_fraction2,
                                number_selected = 50,
                                min_ntree = min_ntree2,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor) # number_selected = 500 if no df cuts of vocabulary is done
###############
#Prediction for P(EPISODE)
set.seed(7)
ff_fit <- ff(X, y, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata

rankings <- ff_fit$feature_list #selected features (generally biased)
final_rf <- ff_fit$final_rf #final rf fitted with selected features


varimp<-final_rf$importance
varimp<-data.frame(varimp)
rownames(varimp)[order(-varimp$MeanDecreaseAccuracy)]
rownames(varimp)[order(-varimp$MeanDecreaseGini)]

#Train Predicted Probability
predprob<-predict(final_rf,X,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y)
#store results in the following variables
horizon2<-tempdata
horizon2$predprob0<-0 #for all
horizon2$predprob1<-0 #for subset1
horizon2$predprob2<-0 #for subset2

horizon2$predoob0<-0 #for all
horizon2$predoob1<-0 #for subset1
horizon2$predoob2<-0 #for subset2

horizon2$predprob0<-predprob[,2]
horizon2$predoob0<-predoob[,2]

###############
# Prediction of tax rise (sign +)
y1<-y[subset1]
X1<-X[subset1,]
set.seed(7)
ff_fit <- ff(X1, y1, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X1,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y1)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y1)
#store results
horizon2$predprob1[subset1]<-predprob[,2]
horizon2$predoob1[subset1]<-predoob[,2]

###############
# Prediction of tax cut (sign -)
y2<-y[subset2]
X2<-X[subset2,]
set.seed(7)
ff_fit <- ff(X2, y2, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X2,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y2)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y2)
#store results
horizon2$predprob2[subset2]<-predprob[,2]
horizon2$predoob2[subset2]<-predoob[,2]


lastline<-rep(0, ncol(horizon2))
horizon2<-rbind(horizon2,lastline,lastline)
horizon2$p1<-horizon2$predprob1/(1-horizon2$predprob1)*(1-horizon2$predprob0)
horizon2$p2<-horizon2$predprob2/(1-horizon2$predprob2)*(1-horizon2$predprob0)


###############################################################
# THREE MONTH AHEAD PREDICTIONS
# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
tempdata<-data[1:(nrow(data)-3),c("news_t","episode_t.3.1","sign_t.3.1","len")] #one period ahead (the last obs I do not have info on t+1 conditional on X t to infer this relation so i removed it)
tempdata2<-data2[1:(nrow(data2)-3),c("news_t","len")]

# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
myCorpus1 <- Corpus(VectorSource(tempdata$news_t))
myCorpus2 <- Corpus(VectorSource(tempdata2$news_t))

# Bounds to build the numerator matrix (which is the leading)
mindf=ceiling(0.10*nrow(tempdata))
maxdf=ceiling(nrow(tempdata))
control=list(bounds=list(global=c(mindf,maxdf)))#,weighting=function(x) weightTfIdf(x, normalize = TRUE))


tdm1 <- TermDocumentMatrix(myCorpus1, control=control)
dictionary2<-as.character(tdm1$dimnames$Terms)
tdm2<-TermDocumentMatrix(myCorpus2,control=list(dictionary=dictionary2))
myCorpus1 <- NULL
myCorpus2<- NULL

# Compute relative frequency of terms of tdm1 wrt tdm2
m1<-as.matrix(tdm1,rownames.force=TRUE)
m2<-as.matrix(tdm2,rownames.force=TRUE)

m1b<-m1[order(rownames(m1)),]
rm(m1,tdm1,myCorpus1) 
m2b<-m2[order(rownames(m2)),]
rm(m2,tdm2,myCorpus2,dictionary2)

m3b<-round(m1b/m2b*100)
m3b[is.nan(m3b)] <- 0 

tdm<-as.TermDocumentMatrix(m3b, weighting = FALSE)
tdm<-removeSparseTerms(tdm, 0.5)
inspect(tdm)
rm(m2b,m1b,m3b,tempdata2)

wordMatrix <- as.data.frame( t(as.matrix(  tdm )))
################################################
# ALL DATA
y<-as.factor(tempdata$episode_t.3.1)
X<-wordMatrix
# SIGN+ subset 1
subset1<-which(tempdata$sign_t.3.1==1 | tempdata$sign_t.3.1 ==0)
# SIGN- subset 2
subset2<-which(tempdata$sign_t.3.1==-1 | tempdata$sign_t.3.1==0)

#1) Module learning using WGCNA (Default, you can customize your own)
#original configuration: minModuleSize<-30;drop_fraction <-.25 
net = blockwiseModules(X, power = 3, minModuleSize = 20,numericLabels = FALSE)
module_membership <- net$colors

#modules description
length(summary.factor(module_membership))
summary.factor(module_membership)
############################################################

mtry_factor <- 1/2; min_ntree <- 5000;  drop_fraction <- .01; ntree_factor <- 500
nodesize <- 1; final_ntree <- 10000; min_ntree2<-5000
mtry_factor2<- 1/2# I do this to control for the number of features from which to sample after screening
#before setting the param above better, hw, we could do better by increasing number of trees last step 
drop_fraction2 <- .01 #the smaller the more iterations it makes in the selection step
screen_params <- screen_control(drop_fraction = drop_fraction,
                                keep_fraction = .25, min_ntree = min_ntree,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor)
select_params <- select_control(drop_fraction = drop_fraction2,
                                number_selected = 50,
                                min_ntree = min_ntree2,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor) # number_selected = 500 if no df cuts of vocabulary is done
###############
#Prediction for P(EPISODE)
set.seed(7)
ff_fit <- ff(X, y, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata

rankings <- ff_fit$feature_list #selected features (generally biased)
final_rf <- ff_fit$final_rf #final rf fitted with selected features


varimp<-final_rf$importance
varimp<-data.frame(varimp)
rownames(varimp)[order(-varimp$MeanDecreaseAccuracy)]
rownames(varimp)[order(-varimp$MeanDecreaseGini)]

#Train Predicted Probability
predprob<-predict(final_rf,X,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y)
#store results in the following variables
horizon3<-tempdata
horizon3$predprob0<-0 #for all
horizon3$predprob1<-0 #for subset1
horizon3$predprob2<-0 #for subset2

horizon3$predoob0<-0 #for all
horizon3$predoob1<-0 #for subset1
horizon3$predoob2<-0 #for subset2


horizon3$predprob0<-predprob[,2]
horizon3$predoob0<-predoob[,2]

###############
# Prediction of tax rise (sign +)
y1<-y[subset1]
X1<-X[subset1,]
set.seed(7)
ff_fit <- ff(X1, y1, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X1,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y1)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y1)
#store results
horizon3$predprob1[subset1]<-predprob[,2]
horizon3$predoob1[subset1]<-predoob[,2]

###############
# Prediction of tax cut (sign -)
y2<-y[subset2]
X2<-X[subset2,]
set.seed(7)
ff_fit <- ff(X2, y2, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X2,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y2)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y2)
#store results
horizon3$predprob2[subset2]<-predprob[,2]
horizon3$predoob2[subset2]<-predoob[,2]


lastline<-rep(0, ncol(horizon3))
horizon3<-rbind(horizon3,lastline,lastline,lastline)
horizon3$p1<-horizon3$predprob1/(1-horizon3$predprob1)*(1-horizon3$predprob0)
horizon3$p2<-horizon3$predprob2/(1-horizon3$predprob2)*(1-horizon3$predprob0)

############################################################
# EXPORT RESULTS 
total<-cbind(data$year,data$month,data$sign_t.1.1,data$sign_t.2.1,data$sign_t.3.1)
total<-cbind(total,horizon1$p1,horizon2$p1,horizon3$p1,horizon1$p2,horizon2$p2,horizon3$p2)
write.csv(total,"analysis/tax_effects/FF_sign_relnewst2_finalR.csv")
#############################################################

