#RF in R
rm(list=setdiff(ls(),"BIGSTART")) #REMOVEL ALL

#1) Import Data
data <-read.csv("./data_mining/cleaned_data/RRNews_episode_nocong2.csv")
data <-data[-1:-3,]
#2) Pre-process data
data <-data[5:nrow(data),]#remove the first 5 observations due to missing news information
cols<-c(data$sign_t.1,data$sign_t.2,data$sign_t.3,data$seconds_t.1,data$seconds_t,data$seconds_t.1,data$seconds_t.2,data$seconds_t.3,data$ss_t.1,data$ss_t.2,data$ss_t.3)
cols<-as.factor(cols)
rownames(data)<-1:nrow(data)
data$news_t <- as.character(data$news_t)

#3) Import Data 2 (all news universe)
data2 <-read.csv("./data_mining/cleaned_data/allvdtv_final.csv")
data2 <-data2[-1:-3,]
data2 <-data2[5:nrow(data2),]#remove the first 5 observations due to missing news information
data2$news_t <- as.character(data2$news_t)

data$len<-nchar(data$news_t)
data2$len<-nchar(data2$news_t)


library(NLP)
library(tm)
#Estimate FRF (Requires randomForest, Wgcna and fuzzyforest)
#############RANDOM FOREST#################
library(randomForest)
######### FUZZY RANDOM FOREST #############
library(fuzzyforest)
library(foreach, doRNG)
library(WGCNA)
set.seed(7)


#####################################################################################################################
#ONE MONTH HORIZON
# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
tempdata<-data[1:(nrow(data)-1),c("news_t","episode_t.1.1","sign_t.1.1","ss_t.1","len","year","month")] #one period ahead (the last obs I do not have info on t+1 conditional on X t to infer this relation so i removed it)
tempdata2<-data2[1:(nrow(data2)-1),c("news_t","len")]

# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
myCorpus1 <- Corpus(VectorSource(tempdata$news_t))
myCorpus2 <- Corpus(VectorSource(tempdata2$news_t))

# Bounds to build the numerator matrix (which is the leading)
mindf=ceiling(0.10*nrow(tempdata))
maxdf=ceiling(nrow(tempdata))
control=list(bounds=list(global=c(mindf,maxdf)))#,weighting=function(x) weightTfIdf(x, normalize = TRUE))


tdm1 <- TermDocumentMatrix(myCorpus1, control=control)
dictionary2<-as.character(tdm1$dimnames$Terms)
tdm2<-TermDocumentMatrix(myCorpus2,control=list(dictionary=dictionary2))
myCorpus1 <- NULL
myCorpus2<- NULL

# Compute relative frequency of terms of tdm1 wrt tdm2
m1<-as.matrix(tdm1,rownames.force=TRUE)
m2<-as.matrix(tdm2,rownames.force=TRUE)

m1b<-m1[order(rownames(m1)),]
rm(m1,tdm1,myCorpus1) 
m2b<-m2[order(rownames(m2)),]
rm(m2,tdm2,myCorpus2,dictionary2)

m3b<-round(m1b/m2b*100)
m3b[is.nan(m3b)] <- 0 

tdm<-as.TermDocumentMatrix(m3b, weighting = FALSE)
tdm<-removeSparseTerms(tdm, 0.5)
inspect(tdm)
rm(m2b,m1b,m3b,tempdata2)

wordMatrix <- as.data.frame( t(as.matrix(  tdm )))

################################################
# ALL DATA
y<-as.factor(tempdata$episode_t.1.1)
X<-wordMatrix
# SS -BIG subset 1
subset1<-which(tempdata$ss_t.1==1 | tempdata$ss_t.1==0)
# SS -SMALL subset 2
subset2<-which(tempdata$ss_t.1==2 | tempdata$ss_t.1==0)
# SS +SMALL subset 3
subset3<-which(tempdata$ss_t.1==3 | tempdata$ss_t.1==0)
# SS +BIG subset 4
subset4<-which(tempdata$ss_t.1==4 | tempdata$ss_t.1==0)

#1) Module learning using WGCNA (Default, you can customize your own)
#original configuration: minModuleSize<-30;drop_fraction <-.25 
net = blockwiseModules(X, power = 3, minModuleSize = 20,numericLabels = FALSE)
module_membership <- net$colors

#modules description
length(summary.factor(module_membership))
summary.factor(module_membership)
############################################################

mtry_factor <- 1/2; min_ntree <- 5000;  drop_fraction <- .01; ntree_factor <- 500
nodesize <- 1; final_ntree <- 10000; min_ntree2<-5000
mtry_factor2<- 1/2# I do this to control for the number of features from which to sample after screening
#before setting the param above better, hw, we could do better by increasing number of trees last step 
drop_fraction2 <- .01 #the smaller the more iterations it makes in the selection step
screen_params <- screen_control(drop_fraction = drop_fraction,
                                keep_fraction = .25, min_ntree = min_ntree,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor)
select_params <- select_control(drop_fraction = drop_fraction2,
                                number_selected = 50,
                                min_ntree = min_ntree2,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor) # number_selected = 500 if no df cuts of vocabulary is done
###############
#Prediction for P(EPISODE)
set.seed(7)
ff_fit <- ff(X, y, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata

rankings <- ff_fit$feature_list #selected features (generally biased)
final_rf <- ff_fit$final_rf #final rf fitted with selected features


varimp<-final_rf$importance
varimp<-data.frame(varimp)
rownames(varimp)[order(-varimp$MeanDecreaseAccuracy)]
rownames(varimp)[order(-varimp$MeanDecreaseGini)]

#Train Predicted Probability
predprob<-predict(final_rf,X,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y)
#store results in the following variables
horizon1<-tempdata
horizon1$predprob0<-0 #for all
horizon1$predprob1<-0 #for subset1
horizon1$predprob2<-0 #for subset2
horizon1$predprob3<-0 
horizon1$predprob4<-0

horizon1$predoob0<-0 #for all
horizon1$predoob1<-0 #for subset1
horizon1$predoob2<-0 #for subset2
horizon1$predoob3<-0
horizon1$predoob4<-0

horizon1$predprob0<-predprob[,2]
horizon1$predoob0<-predoob[,2]

###############
# Prediction of SS1 BIG-
y1<-y[subset1]
X1<-X[subset1,]
set.seed(7)
ff_fit <- ff(X1, y1, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X1,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y1)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y1)
#store results
horizon1$predprob1[subset1]<-predprob[,2]
horizon1$predoob1[subset1]<-predoob[,2]

###############
# Prediction of SS2 SMALL-
y2<-y[subset2]
X2<-X[subset2,]
set.seed(7)
ff_fit <- ff(X2, y2, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X2,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y2)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y2)
#store results
horizon1$predprob2[subset2]<-predprob[,2]
horizon1$predoob2[subset2]<-predoob[,2]


###############
# Prediction of SS3 SMALL+
y3<-y[subset3]
X3<-X[subset3,]
set.seed(7)
ff_fit <- ff(X3, y3, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X3,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y3)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y3)
#store results
horizon1$predprob3[subset3]<-predprob[,2]
horizon1$predoob3[subset3]<-predoob[,2]

###############
# Prediction of SS4 BIG+
y4<-y[subset4]
X4<-X[subset4,]
set.seed(7)
ff_fit <- ff(X4, y4, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X4,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y4)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y4)
#store results
horizon1$predprob4[subset4]<-predprob[,2]
horizon1$predoob4[subset4]<-predoob[,2]



lastline<-rep(0, ncol(horizon1))
horizon1<-rbind(horizon1,lastline)
horizon1$p1<-horizon1$predprob1/(1-horizon1$predprob1)*(1-horizon1$predprob0)
horizon1$p2<-horizon1$predprob2/(1-horizon1$predprob2)*(1-horizon1$predprob0)
horizon1$p3<-horizon1$predprob3/(1-horizon1$predprob3)*(1-horizon1$predprob0)
horizon1$p4<-horizon1$predprob4/(1-horizon1$predprob4)*(1-horizon1$predprob0)

horizon1$oob1<-horizon1$predoob1/(1-horizon1$predoob1)*(1-horizon1$predoob0)
horizon1$oob2<-horizon1$predoob2/(1-horizon1$predoob2)*(1-horizon1$predoob0)
horizon1$oob3<-horizon1$predoob3/(1-horizon1$predoob3)*(1-horizon1$predoob0)
horizon1$oob4<-horizon1$predoob4/(1-horizon1$predoob4)*(1-horizon1$predoob0)


library(zoo)
library(ggplot2)
subset11<-tempdata$ss_t.1
subset11[subset11!=1]<-0
subset22<-tempdata$ss_t.1
subset22[subset22!=2]<-0
subset22[subset22==2]<-1
subset33<-tempdata$ss_t.1
subset33[subset33!=3]<-0
subset33[subset33==3]<-1
subset44<-tempdata$ss_t.1
subset44[subset44!=4]<-0
subset44[subset44==4]<-1


p11oob<-horizon1$oob1[1:(nrow(horizon1)-1)]
p22oob<-horizon1$oob2[1:(nrow(horizon1)-1)]
p33oob<-horizon1$oob3[1:(nrow(horizon1)-1)]
p44oob<-horizon1$oob4[1:(nrow(horizon1)-1)]
p11prob<-horizon1$predprob1[1:(nrow(horizon1)-1)]
p22prob<-horizon1$predprob2[1:(nrow(horizon1)-1)]
p33prob<-horizon1$predprob3[1:(nrow(horizon1)-1)]
p44prob<-horizon1$predprob4[1:(nrow(horizon1)-1)]

final_data<- data.frame(p11oob,p22oob,p33oob,p44oob,p11prob,p22prob,p33prob,p44prob,subset11,subset22,subset33,subset44,tempdata$year,tempdata$month) 

final_data$timemonth<-final_data$tempdata.year+(final_data$tempdata.month-1)/12
final_data$mtime<-as.Date(yearmon(final_data$timemonth))
episodes11<-final_data$mtime[final_data$subset11==1]
episodes22<-final_data$mtime[final_data$subset22==1]
episodes33<-final_data$mtime[final_data$subset33==1]
episodes44<-final_data$mtime[final_data$subset44==1]



###############################################################
###############################################################
# TWO MONTH HORIZON
# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
tempdata<-data[1:(nrow(data)-2),c("news_t","episode_t.2.1","sign_t.2.1","ss_t.2","len")] #two period ahead (the last 2 obs I do not have info on t+2 conditional on X t to infer this relation so i removed it)
tempdata2<-data2[1:(nrow(data2)-2),c("news_t","len")]

# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
myCorpus1 <- Corpus(VectorSource(tempdata$news_t))
myCorpus2 <- Corpus(VectorSource(tempdata2$news_t))

# Bounds to build the numerator matrix (which is the leading)
mindf=ceiling(0.10*nrow(tempdata))
maxdf=ceiling(nrow(tempdata))
control=list(bounds=list(global=c(mindf,maxdf)))#,weighting=function(x) weightTfIdf(x, normalize = TRUE))


tdm1 <- TermDocumentMatrix(myCorpus1, control=control)
dictionary2<-as.character(tdm1$dimnames$Terms)
tdm2<-TermDocumentMatrix(myCorpus2,control=list(dictionary=dictionary2))
myCorpus1 <- NULL
myCorpus2<- NULL

# Compute relative frequency of terms of tdm1 wrt tdm2
m1<-as.matrix(tdm1,rownames.force=TRUE)
m2<-as.matrix(tdm2,rownames.force=TRUE)

m1b<-m1[order(rownames(m1)),]
rm(m1,tdm1,myCorpus1) 
m2b<-m2[order(rownames(m2)),]
rm(m2,tdm2,myCorpus2,dictionary2)

m3b<-round(m1b/m2b*100)
m3b[is.nan(m3b)] <- 0 

tdm<-as.TermDocumentMatrix(m3b, weighting = FALSE)
tdm<-removeSparseTerms(tdm, 0.5)
inspect(tdm)
rm(m2b,m1b,m3b,tempdata2)

wordMatrix <- as.data.frame( t(as.matrix(  tdm )))

################################################
# ALL DATA
y<-as.factor(tempdata$episode_t.2.1)
X<-wordMatrix
# SS -BIG subset 1
subset1<-which(tempdata$ss_t.2==1 | tempdata$ss_t.2==0)
# SS -SMALL subset 2
subset2<-which(tempdata$ss_t.2==2 | tempdata$ss_t.2==0)
# SS +SMALL subset 3
subset3<-which(tempdata$ss_t.2==3 | tempdata$ss_t.2==0)
# SS +BIG subset 4
subset4<-which(tempdata$ss_t.2==4 | tempdata$ss_t.2==0)

#1) Module learning using WGCNA (Default, you can customize your own)
#original configuration: minModuleSize<-30;drop_fraction <-.25 
net = blockwiseModules(X, power = 3, minModuleSize = 20,numericLabels = FALSE)
module_membership <- net$colors

#modules description
length(summary.factor(module_membership))
summary.factor(module_membership)
############################################################

mtry_factor <- 1/2; min_ntree <- 5000;  drop_fraction <- .01; ntree_factor <- 500
nodesize <- 1; final_ntree <- 10000; min_ntree2<-5000
mtry_factor2<- 1/2# I do this to control for the number of features from which to sample after screening
#before setting the param above better, hw, we could do better by increasing number of trees last step 
drop_fraction2 <- .01 #the smaller the more iterations it makes in the selection step
screen_params <- screen_control(drop_fraction = drop_fraction,
                                keep_fraction = .25, min_ntree = min_ntree,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor)
select_params <- select_control(drop_fraction = drop_fraction2,
                                number_selected = 50,
                                min_ntree = min_ntree2,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor) # number_selected = 500 if no df cuts of vocabulary is done
###############
#Prediction for P(EPISODE)
set.seed(7)
ff_fit <- ff(X, y, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata

rankings <- ff_fit$feature_list #selected features (generally biased)
final_rf <- ff_fit$final_rf #final rf fitted with selected features


varimp<-final_rf$importance
varimp<-data.frame(varimp)
rownames(varimp)[order(-varimp$MeanDecreaseAccuracy)]
rownames(varimp)[order(-varimp$MeanDecreaseGini)]

#Train Predicted Probability
predprob<-predict(final_rf,X,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y)
#store results in the following variables
horizon2<-tempdata
horizon2$predprob0<-0 #for all
horizon2$predprob1<-0 #for subset1
horizon2$predprob2<-0 #for subset2
horizon2$predprob3<-0
horizon2$predprob4<-0

horizon2$predoob0<-0 #for all
horizon2$predoob1<-0 #for subset1
horizon2$predoob2<-0 #for subset2
horizon2$predoob3<-0
horizon2$predoob4<-0

horizon2$predprob0<-predprob[,2]
horizon2$predoob0<-predoob[,2]

###############
# Prediction of SS1 BIG-
y1<-y[subset1]
X1<-X[subset1,]
set.seed(7)
ff_fit <- ff(X1, y1, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X1,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y1)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y1)
#store results
horizon2$predprob1[subset1]<-predprob[,2]
horizon2$predoob1[subset1]<-predoob[,2]

###############
# Prediction of SS2 SMALL-
y2<-y[subset2]
X2<-X[subset2,]
set.seed(7)
ff_fit <- ff(X2, y2, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X2,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y2)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y2)
#store results
horizon2$predprob2[subset2]<-predprob[,2]
horizon2$predoob2[subset2]<-predoob[,2]


###############
# Prediction of SS3 SMALL+
y3<-y[subset3]
X3<-X[subset3,]
set.seed(7)
ff_fit <- ff(X3, y3, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X3,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y3)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y3)
#store results
horizon2$predprob3[subset3]<-predprob[,2]
horizon2$predoob3[subset3]<-predoob[,2]

###############
# Prediction of SS4 BIG+
y4<-y[subset4]
X4<-X[subset4,]
set.seed(7)
ff_fit <- ff(X4, y4, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X4,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y4)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y4)
#store results
horizon2$predprob4[subset4]<-predprob[,2]
horizon2$predoob4[subset4]<-predoob[,2]



lastline<-rep(0, ncol(horizon2))
horizon2<-rbind(horizon2,lastline,lastline)
horizon2$p1<-horizon2$predprob1/(1-horizon2$predprob1)*(1-horizon2$predprob0)
horizon2$p2<-horizon2$predprob2/(1-horizon2$predprob2)*(1-horizon2$predprob0)
horizon2$p3<-horizon2$predprob3/(1-horizon2$predprob3)*(1-horizon2$predprob0)
horizon2$p4<-horizon2$predprob4/(1-horizon2$predprob4)*(1-horizon2$predprob0)


###############################################################
# THREE MONTH AHEAD PREDICTIONS
# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
tempdata<-data[1:(nrow(data)-3),c("news_t","episode_t.3.1","sign_t.3.1","ss_t.3","len")] #one period ahead (the last obs I do not have info on t+1 conditional on X t to infer this relation so i removed it)
tempdata2<-data2[1:(nrow(data2)-3),c("news_t","len")]

# Text vectorization and removal of very rare and too common. (to reduce vocab dimensionality)
myCorpus1 <- Corpus(VectorSource(tempdata$news_t))
myCorpus2 <- Corpus(VectorSource(tempdata2$news_t))

# Bounds to build the numerator matrix (which is the leading)
mindf=ceiling(0.10*nrow(tempdata))
maxdf=ceiling(nrow(tempdata))
control=list(bounds=list(global=c(mindf,maxdf)))#,weighting=function(x) weightTfIdf(x, normalize = TRUE))


tdm1 <- TermDocumentMatrix(myCorpus1, control=control)
dictionary2<-as.character(tdm1$dimnames$Terms)
tdm2<-TermDocumentMatrix(myCorpus2,control=list(dictionary=dictionary2))
myCorpus1 <- NULL
myCorpus2<- NULL

# Compute relative frequency of terms of tdm1 wrt tdm2
m1<-as.matrix(tdm1,rownames.force=TRUE)
m2<-as.matrix(tdm2,rownames.force=TRUE)

m1b<-m1[order(rownames(m1)),]
rm(m1,tdm1,myCorpus1) 
m2b<-m2[order(rownames(m2)),]
rm(m2,tdm2,myCorpus2,dictionary2)

m3b<-round(m1b/m2b*100)
m3b[is.nan(m3b)] <- 0 

tdm<-as.TermDocumentMatrix(m3b, weighting = FALSE)
tdm<-removeSparseTerms(tdm, 0.5)
inspect(tdm)
rm(m2b,m1b,m3b,tempdata2)

wordMatrix <- as.data.frame( t(as.matrix(  tdm )))
################################################
# ALL DATA
y<-as.factor(tempdata$episode_t.3.1)
X<-wordMatrix
# SS -BIG subset 1
subset1<-which(tempdata$ss_t.3==1 | tempdata$ss_t.3==0)
# SS -SMALL subset 2
subset2<-which(tempdata$ss_t.3==2 | tempdata$ss_t.3==0)
# SS +SMALL subset 3
subset3<-which(tempdata$ss_t.3==3 | tempdata$ss_t.3==0)
# SS +BIG subset 4
subset4<-which(tempdata$ss_t.3==4 | tempdata$ss_t.3==0)

#1) Module learning using WGCNA (Default, you can customize your own)
#original configuration: minModuleSize<-30;drop_fraction <-.25 
net = blockwiseModules(X, power = 3, minModuleSize = 20,numericLabels = FALSE)
module_membership <- net$colors

#modules description
length(summary.factor(module_membership))
summary.factor(module_membership)
############################################################

mtry_factor <- 1/2; min_ntree <- 5000;  drop_fraction <- .01; ntree_factor <- 500
nodesize <- 1; final_ntree <- 10000; min_ntree2<-5000
mtry_factor2<- 1/2# I do this to control for the number of features from which to sample after screening
#before setting the param above better, hw, we could do better by increasing number of trees last step 
drop_fraction2 <- .01 #the smaller the more iterations it makes in the selection step
screen_params <- screen_control(drop_fraction = drop_fraction,
                                keep_fraction = .25, min_ntree = min_ntree,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor)
select_params <- select_control(drop_fraction = drop_fraction2,
                                number_selected = 50,
                                min_ntree = min_ntree2,
                                ntree_factor = ntree_factor,
                                mtry_factor = mtry_factor) # number_selected = 500 if no df cuts of vocabulary is done
###############
#Prediction for P(EPISODE)
set.seed(7)
ff_fit <- ff(X, y, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata

rankings <- ff_fit$feature_list #selected features (generally biased)
final_rf <- ff_fit$final_rf #final rf fitted with selected features


varimp<-final_rf$importance
varimp<-data.frame(varimp)
rownames(varimp)[order(-varimp$MeanDecreaseAccuracy)]
rownames(varimp)[order(-varimp$MeanDecreaseGini)]

#Train Predicted Probability
predprob<-predict(final_rf,X,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y)
#store results in the following variables
horizon3<-tempdata
horizon3$predprob0<-0 #for all
horizon3$predprob1<-0 #for subset1
horizon3$predprob2<-0 #for subset2
horizon3$predprob3<-0
horizon3$predprob4<-0

horizon3$predoob0<-0 #for all
horizon3$predoob1<-0 #for subset1
horizon3$predoob2<-0 #for subset2
horizon3$predoob3<-0
horizon3$predoob4<-0

horizon3$predprob0<-predprob[,2]
horizon3$predoob0<-predoob[,2]

###############
# Prediction of SS1 BIG-
y1<-y[subset1]
X1<-X[subset1,]
set.seed(7)
ff_fit <- ff(X1, y1, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X1,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y1)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y1)
#store results
horizon3$predprob1[subset1]<-predprob[,2]
horizon3$predoob1[subset1]<-predoob[,2]

###############
# Prediction of SS2 SMALL-
y2<-y[subset2]
X2<-X[subset2,]
set.seed(7)
ff_fit <- ff(X2, y2, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X2,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y2)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y2)
#store results
horizon3$predprob2[subset2]<-predprob[,2]
horizon3$predoob2[subset2]<-predoob[,2]


###############
# Prediction of SS3 SMALL+
y3<-y[subset3]
X3<-X[subset3,]
set.seed(7)
ff_fit <- ff(X3, y3, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X3,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y3)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y3)
#store results
horizon3$predprob3[subset3]<-predprob[,2]
horizon3$predoob3[subset3]<-predoob[,2]

###############
# Prediction of SS4 BIG+
y4<-y[subset4]
X4<-X[subset4,]
set.seed(7)
ff_fit <- ff(X4, y4, module_membership = module_membership,
             screen_params = screen_params, select_params=select_params,
             final_ntree = final_ntree,nodesize = nodesize, num_processors = 4)#Another control is Z=Zdata
final_rf <- ff_fit$final_rf #final rf fitted with selected features
#Train Predicted Probability
predprob<-predict(final_rf,X4,type = "prob") #cbind(X,Zdata)
predprob<-cbind(predprob,y4)
predprob<-data.frame(predprob)
#OOB votes
predoob<-final_rf$votes
predoob<-cbind(predoob,y4)
#store results
horizon3$predprob4[subset4]<-predprob[,2]
horizon3$predoob4[subset4]<-predoob[,2]


lastline<-rep(0, ncol(horizon3))
horizon3<-rbind(horizon3,lastline,lastline,lastline)
horizon3$p1<-horizon3$predprob1/(1-horizon3$predprob1)*(1-horizon3$predprob0)
horizon3$p2<-horizon3$predprob2/(1-horizon3$predprob2)*(1-horizon3$predprob0)
horizon3$p3<-horizon3$predprob3/(1-horizon3$predprob3)*(1-horizon3$predprob0)
horizon3$p4<-horizon3$predprob4/(1-horizon3$predprob4)*(1-horizon3$predprob0)

############################################################
# EXPORT RESULTS 
total<-cbind(data$year,data$month,data$ss_t.1,data$ss_t.2,data$ss_t.3)
total<-cbind(total,horizon1$p1,horizon2$p1,horizon3$p1,horizon1$p2,horizon2$p2,horizon3$p2,horizon1$p3,horizon2$p3,horizon3$p3,horizon1$p4,horizon2$p4,horizon3$p4)
write.csv(total,"analysis/tax_effects/FF_sizesign_relnewst3_finalR.csv")
#############################################################

